@@ -8,7 +8,9 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os, time, re
 from collections import defaultdict
-from future_builtins import map
+from polyglot.builtins import itervalues, map, unicode_type
+from contextlib import contextmanager
+from functools import partial
 
 from calibre import prints
 from calibre.constants import iswindows, isosx, filesystem_encoding
@@ -67,7 +69,7 @@ def metadata_extensions():
     # but not actually added)
     global _metadata_extensions
     if _metadata_extensions is None:
-        _metadata_extensions =  frozenset(map(unicode, BOOK_EXTENSIONS)) | {'opf'}
+        _metadata_extensions =  frozenset(map(unicode_type, BOOK_EXTENSIONS)) | {'opf'}
     return _metadata_extensions
 
 
@@ -105,6 +107,27 @@ def allow_path(path, ext, compiled_rules):
     return ans
 
 
+import_ctx = None
+
+
+@contextmanager
+def run_import_plugins_before_metadata(tdir, group_id=0):
+    global import_ctx
+    import_ctx = {'tdir': tdir, 'group_id': group_id, 'format_map': {}}
+    yield import_ctx
+    import_ctx = None
+
+
+def run_import_plugins(formats):
+    from calibre.ebooks.metadata.worker import run_import_plugins
+    import_ctx['group_id'] += 1
+    ans = run_import_plugins(formats, import_ctx['group_id'], import_ctx['tdir'])
+    fm = import_ctx['format_map']
+    for old_path, new_path in zip(formats, ans):
+        fm[new_path] = old_path
+    return ans
+
+
 def find_books_in_directory(dirpath, single_book_per_directory, compiled_rules=(), listdir_impl=listdir):
     dirpath = os.path.abspath(dirpath)
     if single_book_per_directory:
@@ -114,21 +137,31 @@ def find_books_in_directory(dirpath, single_book_per_directory, compiled_rules=(
             if allow_path(path, ext, compiled_rules):
                 formats[ext] = path
         if formats_ok(formats):
-            yield list(formats.itervalues())
+            yield list(itervalues(formats))
     else:
         books = defaultdict(dict)
         for path in listdir_impl(dirpath, sort_by_mtime=True):
             key, ext = splitext(path)
             if allow_path(path, ext, compiled_rules):
-                books[icu_lower(key) if isinstance(key, unicode) else key.lower()][ext] = path
+                books[icu_lower(key) if isinstance(key, unicode_type) else key.lower()][ext] = path
 
-        for formats in books.itervalues():
+        for formats in itervalues(books):
             if formats_ok(formats):
-                yield list(formats.itervalues())
+                yield list(itervalues(formats))
+
+
+def create_format_map(formats):
+    format_map = {}
+    for path in formats:
+        ext = os.path.splitext(path)[1][1:].upper()
+        if ext == 'OPF':
+            continue
+        format_map[ext] = path
+    return format_map
 
 
 def import_book_directory_multiple(db, dirpath, callback=None,
-        added_ids=None, compiled_rules=()):
+        added_ids=None, compiled_rules=(), add_duplicates=False):
     from calibre.ebooks.metadata.meta import metadata_from_formats
 
     duplicates = []
@@ -136,10 +169,11 @@ def import_book_directory_multiple(db, dirpath, callback=None,
         mi = metadata_from_formats(formats)
         if mi.title is None:
             continue
-        if db.has_book(mi):
+        ids, dups = db.new_api.add_books([(mi, create_format_map(formats))], add_duplicates=add_duplicates)
+        if dups:
             duplicates.append((mi, formats))
             continue
-        book_id = db.import_book(mi, formats)
+        book_id = next(iter(ids))
         if added_ids is not None:
             added_ids.add(book_id)
         if callable(callback):
@@ -148,7 +182,7 @@ def import_book_directory_multiple(db, dirpath, callback=None,
     return duplicates
 
 
-def import_book_directory(db, dirpath, callback=None, added_ids=None, compiled_rules=()):
+def import_book_directory(db, dirpath, callback=None, added_ids=None, compiled_rules=(), add_duplicates=False):
     from calibre.ebooks.metadata.meta import metadata_from_formats
     dirpath = os.path.abspath(dirpath)
     formats = None
@@ -159,9 +193,10 @@ def import_book_directory(db, dirpath, callback=None, added_ids=None, compiled_r
     mi = metadata_from_formats(formats)
     if mi.title is None:
         return
-    if db.has_book(mi):
+    ids, dups = db.new_api.add_books([(mi, create_format_map(formats))], add_duplicates=add_duplicates)
+    if dups:
         return [(mi, formats)]
-    book_id = db.import_book(mi, formats)
+    book_id = next(iter(ids))
     if added_ids is not None:
         added_ids.add(book_id)
     if callable(callback):
@@ -169,14 +204,13 @@ def import_book_directory(db, dirpath, callback=None, added_ids=None, compiled_r
 
 
 def recursive_import(db, root, single_book_per_directory=True,
-        callback=None, added_ids=None, compiled_rules=()):
+        callback=None, added_ids=None, compiled_rules=(), add_duplicates=False):
     root = os.path.abspath(root)
     duplicates  = []
     for dirpath in os.walk(root):
-        res = (import_book_directory(db, dirpath[0], callback=callback,
-            added_ids=added_ids, compiled_rules=compiled_rules) if single_book_per_directory else
-            import_book_directory_multiple(db, dirpath[0],
-                callback=callback, added_ids=added_ids, compiled_rules=compiled_rules))
+        func = import_book_directory if single_book_per_directory else import_book_directory_multiple
+        res = func(db, dirpath[0], callback=callback,
+            added_ids=added_ids, compiled_rules=compiled_rules, add_duplicates=add_duplicates)
         if res is not None:
             duplicates.extend(res)
         if callable(callback):
@@ -185,6 +219,18 @@ def recursive_import(db, root, single_book_per_directory=True,
     return duplicates
 
 
+def cdb_find_in_dir(dirpath, single_book_per_directory, compiled_rules):
+    return find_books_in_directory(dirpath, single_book_per_directory=single_book_per_directory,
+            compiled_rules=compiled_rules, listdir_impl=partial(listdir, sort_by_mtime=True))
+
+
+def cdb_recursive_find(root, single_book_per_directory=True, compiled_rules=()):
+    root = os.path.abspath(root)
+    for dirpath in os.walk(root):
+        for formats in cdb_find_in_dir(dirpath[0], single_book_per_directory, compiled_rules):
+            yield formats
+
+
 def add_catalog(cache, path, title, dbapi=None):
     from calibre.ebooks.metadata.book.base import Metadata
     from calibre.ebooks.metadata.meta import get_metadata
