@@ -7,9 +7,15 @@ import re
 import socket
 import time
 from functools import partial
-from Queue import Empty, Queue
+try:
+    from queue import Empty, Queue
+except ImportError:
+    from Queue import Empty, Queue
 from threading import Thread
-from urlparse import urlparse
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
 
 from calibre import as_unicode, browser, random_user_agent
 from calibre.ebooks.metadata import check_isbn
@@ -83,14 +89,17 @@ def parse_details_page(url, log, timeout, browser, domain):
     if domain == 'jp':
         for a in root.xpath('//a[@href]'):
             if 'black-curtain-redirect.html' in a.get('href'):
-                url = 'https://amazon.co.jp' + a.get('href')
-                log('Black curtain redirect found, following')
-                return parse_details_page(url, log, timeout, browser, domain)
+                url = a.get('href')
+                if url:
+                    if url.startswith('/'):
+                        url = 'https://amazon.co.jp' + a.get('href')
+                    log('Black curtain redirect found, following')
+                    return parse_details_page(url, log, timeout, browser, domain)
 
     errmsg = root.xpath('//*[@id="errorMessage"]')
     if errmsg:
         msg = 'Failed to parse amazon details page: %r' % url
-        msg += tostring(errmsg, method='text', encoding=unicode).strip()
+        msg += tostring(errmsg, method='text', encoding='unicode').strip()
         log.error(msg)
         return
 
@@ -290,7 +299,7 @@ class Worker(Thread):  # Get details {{{
             'chs': ('Chinese', u'中文', u'简体中文'),
         }
         self.lang_map = {}
-        for code, names in lm.iteritems():
+        for code, names in lm.items():
             for name in names:
                 self.lang_map[name] = code
 
@@ -310,7 +319,7 @@ class Worker(Thread):  # Get details {{{
         if not self.months:
             return raw
         ans = raw.lower()
-        for i, vals in self.months.iteritems():
+        for i, vals in self.months.items():
             for x in vals:
                 ans = ans.replace(x, self.english_months[i])
         ans = ans.replace(' de ', ' ')
@@ -344,7 +353,7 @@ class Worker(Thread):  # Get details {{{
             with tempfile.NamedTemporaryFile(prefix=(asin or str(uuid.uuid4())) + '_',
                                              suffix='.html', delete=False) as f:
                 f.write(raw)
-            print ('Downloaded html for', asin, 'saved in', f.name)
+            print('Downloaded html for', asin, 'saved in', f.name)
 
         try:
             title = self.parse_title(root)
@@ -463,7 +472,7 @@ class Worker(Thread):  # Get details {{{
             self.result_queue.put(mi)
 
     def totext(self, elem):
-        return self.tostring(elem, encoding=unicode, method='text').strip()
+        return self.tostring(elem, encoding='unicode', method='text').strip()
 
     def parse_title(self, root):
         h1 = root.xpath('//h1[@id="title"]')
@@ -475,10 +484,10 @@ class Worker(Thread):  # Get details {{{
         tdiv = root.xpath('//h1[contains(@class, "parseasinTitle")]')[0]
         actual_title = tdiv.xpath('descendant::*[@id="btAsinTitle"]')
         if actual_title:
-            title = self.tostring(actual_title[0], encoding=unicode,
+            title = self.tostring(actual_title[0], encoding='unicode',
                                   method='text').strip()
         else:
-            title = self.tostring(tdiv, encoding=unicode,
+            title = self.tostring(tdiv, encoding='unicode',
                                   method='text').strip()
         ans = re.sub(r'[(\[].*[)\]]', '', title).strip()
         if not ans:
@@ -505,7 +514,7 @@ class Worker(Thread):  # Get details {{{
                     ''')
         for x in aname:
             x.tail = ''
-        authors = [self.tostring(x, encoding=unicode, method='text').strip() for x
+        authors = [self.tostring(x, encoding='unicode', method='text').strip() for x
                    in aname]
         authors = [a for a in authors if a]
         return authors
@@ -556,7 +565,7 @@ class Worker(Thread):  # Get details {{{
         for a in desc.xpath('descendant::a[@href]'):
             del a.attrib['href']
             a.tag = 'span'
-        desc = self.tostring(desc, method='html', encoding=unicode).strip()
+        desc = self.tostring(desc, method='html', encoding='unicode').strip()
 
         # Encoding bug in Amazon data U+fffd (replacement char)
         # in some examples it is present in place of '
@@ -573,7 +582,10 @@ class Worker(Thread):  # Get details {{{
         return sanitize_comments_html(desc)
 
     def parse_comments(self, root, raw):
-        from urllib import unquote
+        try:
+            from urllib.parse import unquote
+        except ImportError:
+            from urllib import unquote
         ans = ''
         ns = tuple(self.selector('#bookDescription_feature_div noscript'))
         if ns:
@@ -623,14 +635,14 @@ class Worker(Thread):  # Get details {{{
             spans = series.xpath('./span')
             if spans:
                 raw = self.tostring(
-                    spans[0], encoding=unicode, method='text', with_tail=False).strip()
+                    spans[0], encoding='unicode', method='text', with_tail=False).strip()
                 m = re.search(r'\s+([0-9.]+)$', raw.strip())
                 if m is not None:
                     series_index = float(m.group(1))
                     s = series.xpath('./a[@id="series-page-link"]')
                     if s:
                         series = self.tostring(
-                            s[0], encoding=unicode, method='text', with_tail=False).strip()
+                            s[0], encoding='unicode', method='text', with_tail=False).strip()
                         if series:
                             ans = (series, series_index)
         # This is found on Kindle edition pages on amazon.com
@@ -643,7 +655,7 @@ class Worker(Thread):  # Get details {{{
                     a = span.xpath('./a[@href]')
                     if a:
                         series = self.tostring(
-                            a[0], encoding=unicode, method='text', with_tail=False).strip()
+                            a[0], encoding='unicode', method='text', with_tail=False).strip()
                         if series:
                             ans = (series, series_index)
         # This is found on newer Kindle edition pages on amazon.com
@@ -656,14 +668,14 @@ class Worker(Thread):  # Get details {{{
                     a = b.getparent().xpath('./a[@href]')
                     if a:
                         series = self.tostring(
-                            a[0], encoding=unicode, method='text', with_tail=False).partition('(')[0].strip()
+                            a[0], encoding='unicode', method='text', with_tail=False).partition('(')[0].strip()
                         if series:
                             ans = series, series_index
 
         if ans == (None, None):
             desc = root.xpath('//div[@id="ps-content"]/div[@class="buying"]')
             if desc:
-                raw = self.tostring(desc[0], method='text', encoding=unicode)
+                raw = self.tostring(desc[0], method='text', encoding='unicode')
                 raw = re.sub(r'\s+', ' ', raw)
                 match = self.series_pat.search(raw)
                 if match is not None:
@@ -743,7 +755,7 @@ class Worker(Thread):  # Get details {{{
                     mwidth = 0
                     try:
                         url = None
-                        for iurl, (width, height) in idata.iteritems():
+                        for iurl, (width, height) in idata.items():
                             if width > mwidth:
                                 mwidth = width
                                 url = iurl
@@ -839,7 +851,7 @@ class Worker(Thread):  # Get details {{{
 class Amazon(Source):
 
     name = 'Amazon.com'
-    version = (1, 2, 3)
+    version = (1, 2, 6)
     minimum_calibre_version = (2, 82, 0)
     description = _('Downloads metadata and covers from Amazon')
 
@@ -885,6 +897,14 @@ class Amazon(Source):
                    ' calibre can fetch the Amazon data from many different'
                    ' places where it is cached. Choose the source you prefer.'
                ), choices=SERVERS),
+        Option('use_mobi_asin', 'bool', False, _('Use the MOBI-ASIN for metadata search'),
+               _(
+                   'Enable this option to search for metadata with an'
+                   ' ASIN identifier from the MOBI file at the current country website,'
+                   ' unless any other amazon id is available. Note that if the'
+                   ' MOBI file came from a different Amazon country store, you could get'
+                   ' incorrect results.'
+               )),
     )
 
     def __init__(self, *args, **kwargs):
@@ -946,14 +966,18 @@ class Amazon(Source):
         self.touched_fields = frozenset(tf)
 
     def get_domain_and_asin(self, identifiers, extra_domains=()):
-        for key, val in identifiers.iteritems():
-            key = key.lower()
+        identifiers = {k.lower(): v for k, v in identifiers.items()}
+        for key, val in identifiers.items():
             if key in ('amazon', 'asin'):
                 return 'com', val
             if key.startswith('amazon_'):
                 domain = key.partition('_')[-1]
                 if domain and (domain in self.AMAZON_DOMAINS or domain in extra_domains):
                     return domain, val
+        if self.prefs['use_mobi_asin']:
+            val = identifiers.get('mobi-asin')
+            if val is not None:
+                return self.domain, val
         return None, None
 
     def referrer_for_domain(self, domain=None):
@@ -1042,7 +1066,10 @@ class Amazon(Source):
 
     def create_query(self, log, title=None, authors=None, identifiers={},  # {{{
                      domain=None, for_amazon=True):
-        from urllib import urlencode
+        try:
+            from urllib.parse import urlencode
+        except ImportError:
+            from urllib import urlencode
         if domain is None:
             domain = self.domain
 
@@ -1115,7 +1142,7 @@ class Amazon(Source):
             encode_to = 'latin1'
         encoded_q = dict([(x.encode(encode_to, 'ignore'), y.encode(encode_to,
                                                                    'ignore')) for x, y in
-                          q.iteritems()])
+                          q.items()])
         url = 'https://www.amazon.%s/s/?' % self.get_website_domain(
             domain) + urlencode(encoded_q)
         return url, domain
@@ -1154,8 +1181,11 @@ class Amazon(Source):
                 return False
             return True
 
-        for a in root.xpath(r'//li[starts-with(@id, "result_")]//a[@href and contains(@class, "s-access-detail-page")]'):
-            title = tostring(a, method='text', encoding=unicode)
+        result_links = root.xpath('//div[contains(@class, "s-result-list")]//div[@data-index]//h5//a[@href]')
+        if not result_links:
+            result_links = root.xpath(r'//li[starts-with(@id, "result_")]//a[@href and contains(@class, "s-access-detail-page")]')
+        for a in result_links:
+            title = tostring(a, method='text', encoding='unicode')
             if title_ok(title):
                 url = a.get('href')
                 if url.startswith('/'):
@@ -1171,7 +1201,7 @@ class Amazon(Source):
                     # New amazon markup
                     links = div.xpath('descendant::h3/a[@href]')
                 for a in links:
-                    title = tostring(a, method='text', encoding=unicode)
+                    title = tostring(a, method='text', encoding='unicode')
                     if title_ok(title):
                         url = a.get('href')
                         if url.startswith('/'):
@@ -1186,7 +1216,7 @@ class Amazon(Source):
             for td in root.xpath(
                     r'//div[@id="Results"]/descendant::td[starts-with(@id, "search:Td:")]'):
                 for a in td.xpath(r'descendant::td[@class="dataColumn"]/descendant::a[@href]/span[@class="srTitle"]/..'):
-                    title = tostring(a, method='text', encoding=unicode)
+                    title = tostring(a, method='text', encoding='unicode')
                     if title_ok(title):
                         url = a.get('href')
                         if url.startswith('/'):
@@ -1238,7 +1268,7 @@ class Amazon(Source):
             with tempfile.NamedTemporaryFile(prefix='amazon_results_',
                                              suffix='.html', delete=False) as f:
                 f.write(raw.encode('utf-8'))
-            print ('Downloaded html for results page saved in', f.name)
+            print('Downloaded html for results page saved in', f.name)
 
         matches = []
         found = '<title>404 - ' not in raw
@@ -1386,9 +1416,9 @@ class Amazon(Source):
         if not self.use_search_engine:
             return True
         if title is not None:
-            tokens = {icu_lower(x) for x in title.split() if len(x) > 3}
+            tokens = {icu_lower(x).rstrip(':') for x in title.split() if len(x) > 3}
             if tokens:
-                result_tokens = {icu_lower(x) for x in mi.title.split()}
+                result_tokens = {icu_lower(x).rstrip(':') for x in mi.title.split()}
                 if not tokens.intersection(result_tokens):
                     log('Ignoring result:', mi.title, 'as its title does not match')
                     return False
