@@ -6,14 +6,12 @@ from __future__ import (unicode_literals, division, absolute_import,
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-from future_builtins import zip
+import numbers
+from polyglot.builtins import iteritems, zip, string_or_bytes
 from functools import wraps
 
-try:
-    from cssutils.css import PropertyValue
-except ImportError:
-    raise RuntimeError('You need cssutils >= 0.9.9 for calibre')
-from cssutils import profile as cssprofiles, CSSParser
+from css_parser.css import PropertyValue
+from css_parser import profile as cssprofiles, CSSParser
 from tinycss.fonts3 import parse_font, serialize_font_family
 
 DEFAULTS = {'azimuth': 'center', 'background-attachment': 'scroll',  # {{{
@@ -60,6 +58,7 @@ DEFAULTS = {'azimuth': 'center', 'background-attachment': 'scroll',  # {{{
 EDGES = ('top', 'right', 'bottom', 'left')
 BORDER_PROPS = ('color', 'style', 'width')
 
+
 def normalize_edge(name, cssvalue):
     style = {}
     if isinstance(cssvalue, PropertyValue):
@@ -89,6 +88,7 @@ def normalize_edge(name, cssvalue):
 
 def simple_normalizer(prefix, names, check_inherit=True):
     composition = tuple('%s-%s' %(prefix, n) for n in names)
+
     @wraps(normalize_simple_composition)
     def wrapper(name, cssvalue):
         return normalize_simple_composition(name, cssvalue, composition, check_inherit=check_inherit)
@@ -112,8 +112,10 @@ def normalize_simple_composition(name, cssvalue, composition, check_inherit=True
                     break
     return style
 
+
 font_composition = ('font-style', 'font-variant', 'font-weight', 'font-size', 'line-height', 'font-family')
 
+
 def normalize_font(cssvalue, font_family_as_list=False):
     # See https://developer.mozilla.org/en-US/docs/Web/CSS/font
     composition = font_composition
@@ -126,20 +128,22 @@ def normalize_font(cssvalue, font_family_as_list=False):
         ans = {k:DEFAULTS[k] for k in composition}
         ans.update(parse_font(val))
     if font_family_as_list:
-        if isinstance(ans['font-family'], basestring):
+        if isinstance(ans['font-family'], string_or_bytes):
             ans['font-family'] = [x.strip() for x in ans['font-family'].split(',')]
     else:
-        if not isinstance(ans['font-family'], basestring):
+        if not isinstance(ans['font-family'], string_or_bytes):
             ans['font-family'] = serialize_font_family(ans['font-family'])
     return ans
 
+
 def normalize_border(name, cssvalue):
     style = normalizers['border-' + EDGES[0]]('border-' + EDGES[0], cssvalue)
     vals = style.copy()
     for edge in EDGES[1:]:
-        style.update({k.replace(EDGES[0], edge):v for k, v in vals.iteritems()})
+        style.update({k.replace(EDGES[0], edge):v for k, v in iteritems(vals)})
     return style
 
+
 normalizers = {
     'list-style': simple_normalizer('list-style', ('type', 'position', 'image')),
     'font': lambda prop, v: normalize_font(v),
@@ -160,6 +164,8 @@ SHORTHAND_DEFAULTS = {
 }
 
 _safe_parser = None
+
+
 def safe_parser():
     global _safe_parser
     if _safe_parser is None:
@@ -167,6 +173,7 @@ def safe_parser():
         _safe_parser = CSSParser(loglevel=logging.CRITICAL, validate=False)
     return _safe_parser
 
+
 def normalize_filter_css(props):
     ans = set()
     p = safe_parser()
@@ -179,6 +186,7 @@ def normalize_filter_css(props):
             ans |= set(n(prop, cssvalue))
     return ans
 
+
 def condense_edge(vals):
     edges = {x.name.rpartition('-')[-1]:x.value for x in vals}
     if len(edges) != 4 or set(edges) != {'left', 'top', 'right', 'bottom'}:
@@ -200,6 +208,7 @@ def condense_edge(vals):
             return ce['top']
         return ' '.join(ce[x] for x in ('top', 'left'))
 
+
 def simple_condenser(prefix, func):
     @wraps(func)
     def condense_simple(style, props):
@@ -210,6 +219,7 @@ def simple_condenser(prefix, func):
             style.setProperty(prefix, cp)
     return condense_simple
 
+
 def condense_border(style, props):
     prop_map = {p.name:p for p in props}
     edge_vals = []
@@ -233,6 +243,7 @@ def condense_border(style, props):
             style.removeProperty(prop.name)
         style.setProperty('border', edge_vals[0].value)
 
+
 condensers = {'margin': simple_condenser('margin', condense_edge), 'padding': simple_condenser('padding', condense_edge), 'border': condense_border}
 
 
@@ -243,18 +254,20 @@ def condense_rule(style):
             if prop.name and prop.name.startswith(x):
                 expanded[x].append(prop)
                 break
-    for prefix, vals in expanded.iteritems():
+    for prefix, vals in iteritems(expanded):
         if len(vals) > 1 and {x.priority for x in vals} == {''}:
             condensers[prefix[:-1]](style, vals)
 
+
 def condense_sheet(sheet):
     for rule in sheet.cssRules:
         if rule.type == rule.STYLE_RULE:
             condense_rule(rule.style)
 
+
 def test_normalization(return_tests=False):  # {{{
     import unittest
-    from cssutils import parseStyle
+    from css_parser import parseStyle
     from itertools import product
 
     class TestNormalization(unittest.TestCase):
@@ -267,7 +280,7 @@ def test_normalization(return_tests=False):  # {{{
                 ans.update(expected)
                 return ans
 
-            for raw, expected in {
+            for raw, expected in iteritems({
                 'some_font': {'font-family':'some_font'}, 'inherit':{k:'inherit' for k in font_composition},
                 '1.2pt/1.4 A_Font': {'font-family':'A_Font', 'font-size':'1.2pt', 'line-height':'1.4'},
                 'bad font': {'font-family':'"bad font"'}, '10% serif': {'font-family':'serif', 'font-size':'10%'},
@@ -278,7 +291,7 @@ def test_normalization(return_tests=False):  # {{{
                 {'font-family':'serif', 'font-weight':'bold', 'font-style':'italic', 'font-size':'larger',
                  'line-height':'normal', 'font-variant':'small-caps'},
                 '2em A B': {'font-family': '"A B"', 'font-size': '2em'},
-            }.iteritems():
+            }):
                 val = tuple(parseStyle('font: %s' % raw, validate=False))[0].cssValue
                 style = normalizers['font']('font', val)
                 self.assertDictEqual(font_dict(expected), style, raw)
@@ -286,53 +299,55 @@ def test_normalization(return_tests=False):  # {{{
         def test_border_normalization(self):
             def border_edge_dict(expected, edge='right'):
                 ans = {'border-%s-%s' % (edge, x): DEFAULTS['border-%s-%s' % (edge, x)] for x in ('style', 'width', 'color')}
-                for x, v in expected.iteritems():
+                for x, v in iteritems(expected):
                     ans['border-%s-%s' % (edge, x)] = v
                 return ans
+
             def border_dict(expected):
                 ans = {}
                 for edge in EDGES:
                     ans.update(border_edge_dict(expected, edge))
                 return ans
+
             def border_val_dict(expected, val='color'):
                 ans = {'border-%s-%s' % (edge, val): DEFAULTS['border-%s-%s' % (edge, val)] for edge in EDGES}
                 for edge in EDGES:
                     ans['border-%s-%s' % (edge, val)] = expected
                 return ans
 
-            for raw, expected in {
+            for raw, expected in iteritems({
                 'solid 1px red': {'color':'red', 'width':'1px', 'style':'solid'},
                 '1px': {'width': '1px'}, '#aaa': {'color': '#aaa'},
                 '2em groove': {'width':'2em', 'style':'groove'},
-            }.iteritems():
+            }):
                 for edge in EDGES:
                     br = 'border-%s' % edge
                     val = tuple(parseStyle('%s: %s' % (br, raw), validate=False))[0].cssValue
                     self.assertDictEqual(border_edge_dict(expected, edge), normalizers[br](br, val))
 
-            for raw, expected in {
+            for raw, expected in iteritems({
                 'solid 1px red': {'color':'red', 'width':'1px', 'style':'solid'},
                 '1px': {'width': '1px'}, '#aaa': {'color': '#aaa'},
                 'thin groove': {'width':'thin', 'style':'groove'},
-            }.iteritems():
+            }):
                 val = tuple(parseStyle('%s: %s' % ('border', raw), validate=False))[0].cssValue
                 self.assertDictEqual(border_dict(expected), normalizers['border']('border', val))
 
-            for name, val in {
+            for name, val in iteritems({
                 'width': '10%', 'color': 'rgb(0, 1, 1)', 'style': 'double',
-            }.iteritems():
+            }):
                 cval = tuple(parseStyle('border-%s: %s' % (name, val), validate=False))[0].cssValue
                 self.assertDictEqual(border_val_dict(val, name), normalizers['border-'+name]('border-'+name, cval))
 
         def test_edge_normalization(self):
             def edge_dict(prefix, expected):
                 return {'%s-%s' % (prefix, edge) : x for edge, x in zip(EDGES, expected)}
-            for raw, expected in {
+            for raw, expected in iteritems({
                 '2px': ('2px', '2px', '2px', '2px'),
                 '1em 2em': ('1em', '2em', '1em', '2em'),
                 '1em 2em 3em': ('1em', '2em', '3em', '2em'),
                 '1 2 3 4': ('1', '2', '3', '4'),
-            }.iteritems():
+            }):
                 for prefix in ('margin', 'padding'):
                     cval = tuple(parseStyle('%s: %s' % (prefix, raw), validate=False))[0].cssValue
                     self.assertDictEqual(edge_dict(prefix, expected), normalizers[prefix](prefix, cval))
@@ -340,14 +355,14 @@ def test_normalization(return_tests=False):  # {{{
         def test_list_style_normalization(self):
             def ls_dict(expected):
                 ans = {'list-style-%s' % x : DEFAULTS['list-style-%s' % x] for x in ('type', 'image', 'position')}
-                for k, v in expected.iteritems():
+                for k, v in iteritems(expected):
                     ans['list-style-%s' % k] = v
                 return ans
-            for raw, expected in {
+            for raw, expected in iteritems({
                 'url(http://www.example.com/images/list.png)': {'image': 'url(http://www.example.com/images/list.png)'},
                 'inside square': {'position':'inside', 'type':'square'},
                 'upper-roman url(img) outside': {'position':'outside', 'type':'upper-roman', 'image':'url(img)'},
-            }.iteritems():
+            }):
                 cval = tuple(parseStyle('list-style: %s' % raw, validate=False))[0].cssValue
                 self.assertDictEqual(ls_dict(expected), normalizers['list-style']('list-style', cval))
 
@@ -367,7 +382,7 @@ def test_normalization(return_tests=False):  # {{{
             ae({'list-style', 'list-style-image', 'list-style-type', 'list-style-position'}, normalize_filter_css({'list-style'}))
 
         def test_edge_condensation(self):
-            for s, v in {
+            for s, v in iteritems({
                 (1, 1, 3) : None,
                 (1, 2, 3, 4) : '2pt 3pt 4pt 1pt',
                 (1, 2, 3, 2) : '2pt 3pt 2pt 1pt',
@@ -376,10 +391,10 @@ def test_normalization(return_tests=False):  # {{{
                 (1, 1, 1, 1) : '1pt',
                 ('2%', '2%', '2%', '2%') : '2%',
                 tuple('0 0 0 0'.split()) : '0',
-            }.iteritems():
+            }):
                 for prefix in ('margin', 'padding'):
-                    css = {'%s-%s' % (prefix, x) : str(y)+'pt' if isinstance(y, (int, float)) else y for x, y in zip(('left', 'top', 'right', 'bottom'), s)}
-                    css = '; '.join(('%s:%s' % (k, v) for k, v in css.iteritems()))
+                    css = {'%s-%s' % (prefix, x) : str(y)+'pt' if isinstance(y, numbers.Number) else y for x, y in zip(('left', 'top', 'right', 'bottom'), s)}
+                    css = '; '.join(('%s:%s' % (k, v) for k, v in iteritems(css)))
                     style = parseStyle(css)
                     condense_rule(style)
                     val = getattr(style.getProperty(prefix), 'value', None)
@@ -416,5 +431,6 @@ def test_normalization(return_tests=False):  # {{{
     unittest.TextTestRunner(verbosity=4).run(tests)
 # }}}
 
+
 if __name__ == '__main__':
     test_normalization()
