@@ -8,10 +8,10 @@ __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import re
-from itertools import izip, groupby
+from itertools import groupby
 from operator import itemgetter
 from collections import Counter, OrderedDict
-from polyglot.builtins import map
+from polyglot.builtins import iteritems, map, zip, unicode_type
 
 from calibre import as_unicode
 from calibre.ebooks.pdf.render.common import (Array, String, Stream,
@@ -62,7 +62,10 @@ class FontStream(Stream):
 
 
 def to_hex_string(c):
-    return bytes(hex(int(c))[2:]).rjust(4, b'0').decode('ascii')
+    ans = hex(int(c))[2:]
+    if isinstance(ans, bytes):
+        ans = ans.decode('ascii')
+    return ans.rjust(4, '0')
 
 
 class CMap(Stream):
@@ -106,8 +109,12 @@ class CMap(Stream):
             maps.append(current_map)
         mapping = []
         for m in maps:
-            meat = '\n'.join('%s %s'%(k, v) for k, v in m.iteritems())
+            meat = '\n'.join('%s %s'%(k, v) for k, v in iteritems(m))
             mapping.append('%d beginbfchar\n%s\nendbfchar'%(len(m), meat))
+        try:
+            name = name.encode('ascii').decode('ascii')
+        except Exception:
+            name = uuid4()
         self.write(self.skeleton.format(name=name, mapping='\n'.join(mapping)))
 
 
@@ -116,8 +123,9 @@ class Font(object):
     def __init__(self, metrics, num, objects, compress):
         self.metrics, self.compress = metrics, compress
         self.is_otf = self.metrics.is_otf
-        self.subset_tag = bytes(re.sub('.', lambda m: chr(int(m.group())+ord('A')),
-                                  oct(num))).rjust(6, b'A').decode('ascii')
+        self.subset_tag = unicode_type(
+            re.sub('.', lambda m: chr(int(m.group())+ord('A')), oct(num).replace('o', '')
+        )).rjust(6, 'A')
         self.font_stream = FontStream(metrics.is_otf, compress=compress)
         try:
             psname = metrics.postscript_name
@@ -184,24 +192,23 @@ class Font(object):
         try:
             name = self.metrics.postscript_name
         except KeyError:
-            from calibre.utils.short_uuid import uuid4
             name = uuid4()
         cmap = CMap(name, self.metrics.glyph_map, compress=self.compress)
         self.font_dict['ToUnicode'] = objects.add(cmap)
 
     def write_widths(self, objects):
         glyphs = sorted(self.used_glyphs|{0})
-        widths = {g:self.metrics.pdf_scale(w) for g, w in izip(glyphs,
+        widths = {g:self.metrics.pdf_scale(w) for g, w in zip(glyphs,
                                         self.metrics.glyph_widths(glyphs))}
         counter = Counter()
-        for g, w in widths.iteritems():
+        for g, w in iteritems(widths):
             counter[w] += 1
         most_common = counter.most_common(1)[0][0]
         self.descendant_font['DW'] = most_common
-        widths = {g:w for g, w in widths.iteritems() if w != most_common}
+        widths = {g:w for g, w in iteritems(widths) if w != most_common}
 
         groups = Array()
-        for k, g in groupby(enumerate(widths.iterkeys()), lambda i_x:i_x[0]-i_x[1]):
+        for k, g in groupby(enumerate(widths), lambda i_x:i_x[0]-i_x[1]):
             group = list(map(itemgetter(1), g))
             gwidths = [widths[g] for g in group]
             if len(set(gwidths)) == 1 and len(group) > 1:
@@ -236,7 +243,7 @@ class FontManager(object):
         if name not in STANDARD_FONTS:
             raise ValueError('%s is not a standard font'%name)
         if name not in self.std_map:
-                self.std_map[name] = self.objects.add(Dictionary({
+            self.std_map[name] = self.objects.add(Dictionary({
                 'Type':Name('Font'),
                 'Subtype':Name('Type1'),
                 'BaseFont':Name(name)
