@@ -68,21 +68,27 @@ def Handle_Main():
     except ParseRtf.RtfInvalidCodeException, msg:
         sys.stderr.write(msg)
 """
+
+
 class InvalidRtfException(Exception):
     """
     handle invalid RTF
     """
     pass
+
+
 class RtfInvalidCodeException(Exception):
     """
     handle bugs in program
     """
     pass
 
+
 class ParseRtf:
     """
     Main class for controlling the rest of the parsing.
     """
+
     def __init__(self,
                 in_file,
                 out_file='',
@@ -224,7 +230,7 @@ class ParseRtf:
             )
         try:
             return_value = process_tokens_obj.process_tokens()
-        except InvalidRtfException, msg:
+        except InvalidRtfException as msg:
             # Check to see if the file is correctly encoded
             encode_obj = default_encoding.DefaultEncoding(
             in_file=self.__temp_file,
@@ -242,7 +248,7 @@ class ParseRtf:
             enc = 'cp' + enc
             msg = '%s\nException in token processing' % str(msg)
             if check_encoding_obj.check_encoding(self.__file, enc):
-                file_name = self.__file if isinstance(self.__file, str) \
+                file_name = self.__file if isinstance(self.__file, bytes) \
                                     else self.__file.encode('utf-8')
                 msg +='\nFile %s does not appear to be correctly encoded.\n' % file_name
             try:
