@@ -7,23 +7,26 @@ __docformat__ = 'restructuredtext en'
 
 import functools
 
-from PyQt5.Qt import (Qt, QApplication, QStackedWidget, QMenu, QTimer,
-        QSizePolicy, QStatusBar, QLabel, QFont, QAction, QTabBar, QStyle,
-        QVBoxLayout, QWidget, QSplitter, QToolButton, QIcon, QPainter, QStyleOption)
+from PyQt5.Qt import (
+    QAction, QApplication, QFont, QIcon, QLabel, QMenu, QPainter, QSizePolicy,
+    QSplitter, QStackedWidget, QStatusBar, QStyle, QStyleOption, Qt, QTabBar, QTimer,
+    QToolButton, QVBoxLayout, QWidget
+)
 
+from calibre.constants import __appname__, get_version, isosx
+from calibre.customize.ui import find_plugin
+from calibre.gui2 import config, error_dialog, gprefs, is_widescreen, open_url
+from calibre.gui2.book_details import BookDetails
+from calibre.gui2.layout_menu import LayoutMenu
+from calibre.gui2.library.alternate_views import GridView
+from calibre.gui2.library.views import BooksView, DeviceBooksView
+from calibre.gui2.notify import get_notifier
+from calibre.gui2.tag_browser.ui import TagBrowserWidget
+from calibre.gui2.widgets import LayoutButton, Splitter
 from calibre.utils.config import prefs
 from calibre.utils.icu import sort_key
-from calibre.constants import (isosx, __appname__, preferred_encoding,
-    get_version)
-from calibre.gui2 import config, is_widescreen, gprefs, error_dialog, open_url
-from calibre.gui2.library.views import BooksView, DeviceBooksView
-from calibre.gui2.library.alternate_views import GridView
-from calibre.gui2.widgets import Splitter, LayoutButton
-from calibre.gui2.tag_browser.ui import TagBrowserWidget
-from calibre.gui2.book_details import BookDetails
-from calibre.gui2.notify import get_notifier
-from calibre.gui2.layout_menu import LayoutMenu
-from calibre.customize.ui import find_plugin
+from calibre.utils.localization import localize_website_link
+from polyglot.builtins import unicode_type
 
 _keep_refs = []
 
@@ -41,6 +44,7 @@ class LibraryViewMixin(object):  # {{{
 
     def init_library_view_mixin(self, db):
         self.library_view.files_dropped.connect(self.iactions['Add Books'].files_dropped, type=Qt.QueuedConnection)
+        self.library_view.books_dropped.connect(self.iactions['Edit Metadata'].books_dropped, type=Qt.QueuedConnection)
         self.library_view.add_column_signal.connect(partial(self.iactions['Preferences'].do_config,
             initial_plugin=('Interface', 'Custom Columns')),
                 type=Qt.QueuedConnection)
@@ -82,6 +86,9 @@ class LibraryViewMixin(object):  # {{{
         populate_menu(dm, gprefs['action-layout-context-menu-device'], self.iactions)
         ec = self.iactions['Edit Collections'].qaction
         self.library_view.set_context_menu(lm, ec)
+        sm = QMenu(self)
+        populate_menu(sm, gprefs['action-layout-context-menu-split'], self.iactions)
+        self.library_view.pin_view.set_context_menu(sm)
         for v in (self.memory_view, self.card_a_view, self.card_b_view):
             v.set_context_menu(dm, ec)
 
@@ -159,7 +166,7 @@ class LibraryWidget(Splitter):  # {{{
         parent.library_view.setObjectName('library_view')
         stack = QStackedWidget(self)
         av = parent.library_view.alternate_views
-        av.set_stack(stack)
+        parent.pin_container = av.set_stack(stack)
         parent.grid_view = GridView(parent)
         parent.grid_view.setObjectName('grid_view')
         av.add_view('grid', parent.grid_view)
@@ -223,7 +230,7 @@ class VersionLabel(QLabel):  # {{{
         self.setToolTip(_('See what\'s new in this calibre release'))
 
     def mouseReleaseEvent(self, ev):
-        open_url('https://calibre-ebook.com/whats-new')
+        open_url(localize_website_link('https://calibre-ebook.com/whats-new'))
         ev.accept()
         return QLabel.mouseReleaseEvent(self, ev)
 
@@ -317,11 +324,6 @@ class StatusBar(QStatusBar):  # {{{
     def show_message(self, msg, timeout=0, show_notification=True):
         self.showMessage(msg, timeout)
         if self.notifier is not None and not config['disable_tray_notification'] and show_notification:
-            if isosx and isinstance(msg, unicode):
-                try:
-                    msg = msg.encode(preferred_encoding)
-                except UnicodeEncodeError:
-                    msg = msg.encode('utf-8')
             self.notifier(msg)
 
     def clear_message(self):
@@ -338,7 +340,7 @@ class GridViewButton(LayoutButton):  # {{{
         self.set_state_to_show()
         self.action_toggle = QAction(self.icon(), _('Toggle') + ' ' + self.label, self)
         gui.addAction(self.action_toggle)
-        gui.keyboard.register_shortcut('grid view toggle' + self.label, unicode(self.action_toggle.text()),
+        gui.keyboard.register_shortcut('grid view toggle' + self.label, unicode_type(self.action_toggle.text()),
                                     default_keys=(sc,), action=self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
         self.action_toggle.changed.connect(self.update_shortcut)
@@ -368,7 +370,7 @@ class SearchBarButton(LayoutButton):  # {{{
         self.set_state_to_hide()
         self.action_toggle = QAction(self.icon(), _('Toggle') + ' ' + self.label, self)
         gui.addAction(self.action_toggle)
-        gui.keyboard.register_shortcut('search bar toggle' + self.label, unicode(self.action_toggle.text()),
+        gui.keyboard.register_shortcut('search bar toggle' + self.label, unicode_type(self.action_toggle.text()),
                                     default_keys=(sc,), action=self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
         self.action_toggle.changed.connect(self.update_shortcut)
@@ -396,7 +398,7 @@ class VLTabs(QTabBar):  # {{{
         self.setDocumentMode(True)
         self.setDrawBase(False)
         self.setMovable(True)
-        self.setTabsClosable(True)
+        self.setTabsClosable(gprefs['vl_tabs_closable'])
         self.gui = parent
         self.ignore_tab_changed = False
         self.currentChanged.connect(self.tab_changed)
@@ -432,17 +434,34 @@ class VLTabs(QTabBar):  # {{{
         gprefs['show_vl_tabs'] = False
         self.setVisible(False)
 
+    def lock_tab(self):
+        gprefs['vl_tabs_closable'] = False
+        self.setTabsClosable(False)
+
+    def unlock_tab(self):
+        gprefs['vl_tabs_closable'] = True
+        self.setTabsClosable(True)
+        try:
+            self.tabButton(0, self.RightSide).setVisible(False)
+        except AttributeError:
+            try:
+                self.tabButton(0, self.LeftSide).setVisible(False)
+            except AttributeError:
+                # On some OS X machines (using native style) the tab button is
+                # on the left
+                pass
+
     def tab_changed(self, idx):
         if self.ignore_tab_changed:
             return
-        vl = unicode(self.tabData(idx) or '').strip() or None
+        vl = unicode_type(self.tabData(idx) or '').strip() or None
         self.gui.apply_virtual_library(vl, update_tabs=False)
 
     def tab_moved(self, from_, to):
-        self.current_db.new_api.set_pref('virt_libs_order', [unicode(self.tabData(i) or '') for i in range(self.count())])
+        self.current_db.new_api.set_pref('virt_libs_order', [unicode_type(self.tabData(i) or '') for i in range(self.count())])
 
     def tab_close(self, index):
-        vl = unicode(self.tabData(index) or '')
+        vl = unicode_type(self.tabData(index) or '')
         if vl:  # Dont allow closing the All Books tab
             self.current_db.new_api.set_pref('virt_libs_hidden', list(
                 self.current_db.prefs['virt_libs_hidden']) + [vl])
@@ -505,16 +524,20 @@ class VLTabs(QTabBar):  # {{{
 
     def contextMenuEvent(self, ev):
         m = QMenu(self)
-        m.addAction(_('Sort alphabetically'), self.sort_alphabetically)
+        m.addAction(_('Sort tabs alphabetically'), self.sort_alphabetically)
         hidden = self.current_db.prefs['virt_libs_hidden']
         if hidden:
             s = m._s = m.addMenu(_('Restore hidden tabs'))
             for x in hidden:
                 s.addAction(x, partial(self.restore, x))
         m.addAction(_('Hide virtual library tabs'), self.disable_bar)
+        if gprefs['vl_tabs_closable']:
+            m.addAction(_('Lock virtual library tabs'), self.lock_tab)
+        else:
+            m.addAction(_('Unlock virtual library tabs'), self.unlock_tab)
         i = self.tabAt(ev.pos())
         if i > -1:
-            vl = unicode(self.tabData(i) or '')
+            vl = unicode_type(self.tabData(i) or '')
             if vl:
                 m.addSeparator()
                 m.addAction(_('Edit "%s"') % vl, partial(self.gui.do_create_edit, name=vl))
@@ -578,7 +601,7 @@ class LayoutMixin(object):  # {{{
             self.qv = self.qv.actual_plugin_
 
         self.status_bar = StatusBar(self)
-        stylename = unicode(self.style().objectName())
+        stylename = unicode_type(self.style().objectName())
         self.grid_view_button = GridViewButton(self)
         self.search_bar_button = SearchBarButton(self)
         self.grid_view_button.toggled.connect(self.toggle_grid_view)
@@ -633,6 +656,8 @@ class LayoutMixin(object):  # {{{
                 type=Qt.QueuedConnection)
         self.book_details.open_fmt_with.connect(self.bd_open_fmt_with,
                 type=Qt.QueuedConnection)
+        self.book_details.edit_book.connect(self.bd_edit_book,
+                type=Qt.QueuedConnection)
         self.book_details.cover_removed.connect(self.bd_cover_removed,
                 type=Qt.QueuedConnection)
         self.book_details.remote_file_dropped.connect(
@@ -656,6 +681,7 @@ class LayoutMixin(object):  # {{{
         self.book_details.view_device_book.connect(
                 self.iactions['View'].view_device_book)
         self.book_details.manage_category.connect(self.manage_category_triggerred)
+        self.book_details.edit_identifiers.connect(self.edit_identifiers_triggerred)
         self.book_details.compare_specific_format.connect(self.compare_format)
 
         m = self.library_view.model()
@@ -665,6 +691,17 @@ class LayoutMixin(object):  # {{{
                     self.library_view.currentIndex())
         self.library_view.setFocus(Qt.OtherFocusReason)
 
+    def edit_identifiers_triggerred(self):
+        book_id = self.library_view.current_book
+        db = self.current_db.new_api
+        identifiers = db.field_for('identifiers', book_id, default_value={})
+        from calibre.gui2.metadata.basic_widgets import Identifiers
+        d = Identifiers(identifiers, self)
+        if d.exec_() == d.Accepted:
+            identifiers = d.get_identifiers()
+            db.set_field('identifiers', {book_id: identifiers})
+            self.iactions['Edit Metadata'].refresh_books_after_metadata_edit({book_id})
+
     def manage_category_triggerred(self, field, value):
         if field and value:
             if field == 'authors':
@@ -704,6 +741,11 @@ class LayoutMixin(object):  # {{{
                     self.current_db.new_api.field_for('title', book_id, default_value=_('Unknown')),
                     fmt), show=True)
 
+    def bd_edit_book(self, book_id, fmt):
+        from calibre.gui2.device import BusyCursor
+        with BusyCursor():
+            self.iactions['Tweak ePub'].ebook_edit_format(book_id, fmt)
+
     def open_with_action_triggerred(self, fmt, entry, *args):
         book_id = self.library_view.current_book
         if book_id is not None:
