@@ -5,11 +5,10 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import json
+import json, textwrap
 
 from collections import defaultdict
 from threading import Thread
-from functools import partial
 
 from PyQt5.Qt import (
     QApplication, QFont, QFontInfo, QFontDialog, QColorDialog, QPainter,
@@ -20,6 +19,7 @@ from PyQt5.Qt import (
 
 from calibre import human_readable
 from calibre.ebooks.metadata.book.render import DEFAULT_AUTHOR_LINK
+from calibre.constants import isosx, iswindows
 from calibre.ebooks.metadata.sources.prefs import msprefs
 from calibre.gui2 import default_author_link
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
@@ -31,9 +31,12 @@ from calibre.utils.localization import (available_translations,
 from calibre.utils.config import prefs
 from calibre.utils.icu import sort_key
 from calibre.gui2.book_details import get_field_list
+from calibre.gui2.dialogs.quickview import get_qv_field_list
 from calibre.gui2.preferences.coloring import EditRules
 from calibre.gui2.library.alternate_views import auto_height, CM_TO_INCH
 from calibre.gui2.widgets2 import Dialog
+from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+from polyglot.builtins import iteritems, unicode_type
 
 
 class BusyCursor(object):
@@ -61,16 +64,19 @@ class DefaultAuthorLink(QWidget):  # {{{
         c.setMinimumContentsLength(30)
         for text, data in [
                 (_('Search for the author on Goodreads'), 'search-goodreads'),
+                (_('Search for the author on Amazon'), 'search-amzn'),
                 (_('Search for the author in your calibre library'), 'search-calibre'),
                 (_('Search for the author on Wikipedia'), 'search-wikipedia'),
                 (_('Search for the author on Google Books'), 'search-google'),
                 (_('Search for the book on Goodreads'), 'search-goodreads-book'),
+                (_('Search for the book on Amazon'), 'search-amzn-book'),
                 (_('Search for the book on Google Books'), 'search-google-book'),
                 (_('Use a custom search URL'), 'url'),
         ]:
             c.addItem(text, data)
         l.addRow(_('Clicking on &author names should:'), c)
         self.custom_url = u = QLineEdit(self)
+        u.textChanged.connect(self.changed_signal)
         u.setPlaceholderText(_('Enter the URL'))
         c.currentIndexChanged.connect(self.current_changed)
         l.addRow(u)
@@ -105,7 +111,7 @@ class IdLinksRuleEdit(Dialog):
     def __init__(self, key='', name='', template='', parent=None):
         title = _('Edit rule') if key else _('Create a new rule')
         Dialog.__init__(self, title=title, name='id-links-rule-editor', parent=parent)
-        self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'http://example.com/{id}')
+        self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'https://example.com/{id}')
 
     @property
     def rule(self):
@@ -153,7 +159,7 @@ class IdLinksEditor(Dialog):
         la.setWordWrap(True)
         l.addWidget(la)
         items = []
-        for k, lx in msprefs['id_link_rules'].iteritems():
+        for k, lx in iteritems(msprefs['id_link_rules']):
             for n, t in lx:
                 items.append((k, n, t))
         items.sort(key=lambda x:sort_key(x[1]))
@@ -166,13 +172,13 @@ class IdLinksEditor(Dialog):
         l.addWidget(t)
         t.horizontalHeader().setSectionResizeMode(2, t.horizontalHeader().Stretch)
         self.cb = b = QPushButton(QIcon(I('plus.png')), _('&Add rule'), self)
-        b.clicked.connect(lambda : self.edit_rule())
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.rb = b = QPushButton(QIcon(I('minus.png')), _('&Remove rule'), self)
-        b.clicked.connect(lambda : self.remove_rule())
+        connect_lambda(b.clicked, self, lambda self: self.remove_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.eb = b = QPushButton(QIcon(I('modified.png')), _('&Edit rule'), self)
-        b.clicked.connect(lambda : self.edit_rule(self.table.currentRow()))
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule(self.table.currentRow()))
         self.bb.addButton(b, self.bb.ActionRole)
         l.addWidget(self.bb)
 
@@ -211,18 +217,20 @@ class IdLinksEditor(Dialog):
 
 class DisplayedFields(QAbstractListModel):  # {{{
 
-    def __init__(self, db, parent=None):
+    def __init__(self, db, parent=None, pref_name=None):
+        self.pref_name = pref_name or 'book_display_fields'
         QAbstractListModel.__init__(self, parent)
 
         self.fields = []
         self.db = db
         self.changed = False
 
+    def get_field_list(self, use_defaults=False):
+        return get_field_list(self.db.field_metadata, use_defaults=use_defaults, pref_name=self.pref_name)
+
     def initialize(self, use_defaults=False):
         self.beginResetModel()
-        self.fields = [[x[0], x[1]] for x in
-                get_field_list(self.db.field_metadata,
-                    use_defaults=use_defaults)]
+        self.fields = [[x[0], x[1]] for x in self.get_field_list(use_defaults=use_defaults)]
         self.endResetModel()
         self.changed = True
 
@@ -249,6 +257,12 @@ class DisplayedFields(QAbstractListModel):  # {{{
             return QIcon(I('column.png'))
         return None
 
+    def toggle_all(self, show=True):
+        for i in range(self.rowCount()):
+            idx = self.index(i)
+            if idx.isValid():
+                self.setData(idx, show, Qt.CheckStateRole)
+
     def flags(self, index):
         ans = QAbstractListModel.flags(self, index)
         return ans | Qt.ItemIsUserCheckable
@@ -267,7 +281,7 @@ class DisplayedFields(QAbstractListModel):  # {{{
 
     def commit(self):
         if self.changed:
-            self.db.new_api.set_pref('book_display_fields', self.fields)
+            self.db.new_api.set_pref(self.pref_name, self.fields)
 
     def move(self, idx, delta):
         row = idx.row() + delta
@@ -281,6 +295,45 @@ class DisplayedFields(QAbstractListModel):  # {{{
             self.changed = True
             return idx
 
+
+def move_field_up(widget, model):
+    idx = widget.currentIndex()
+    if idx.isValid():
+        idx = model.move(idx, -1)
+        if idx is not None:
+            sm = widget.selectionModel()
+            sm.select(idx, sm.ClearAndSelect)
+            widget.setCurrentIndex(idx)
+
+
+def move_field_down(widget, model):
+    idx = widget.currentIndex()
+    if idx.isValid():
+        idx = model.move(idx, 1)
+        if idx is not None:
+            sm = widget.selectionModel()
+            sm.select(idx, sm.ClearAndSelect)
+            widget.setCurrentIndex(idx)
+
+# }}}
+
+
+class QVDisplayedFields(DisplayedFields):  # {{{
+
+    def __init__(self, db, parent=None):
+        DisplayedFields.__init__(self, db, parent)
+
+    def initialize(self, use_defaults=False):
+        self.beginResetModel()
+        self.fields = [[x[0], x[1]] for x in
+                get_qv_field_list(self.db.field_metadata, use_defaults=use_defaults)]
+        self.endResetModel()
+        self.changed = True
+
+    def commit(self):
+        if self.changed:
+            self.db.new_api.set_pref('qv_display_fields', self.fields)
+
 # }}}
 
 
@@ -324,6 +377,10 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
 
     def genesis(self, gui):
         self.gui = gui
+        if not isosx and not iswindows:
+            self.label_widget_style.setVisible(False)
+            self.opt_ui_style.setVisible(False)
+
         db = gui.library_view.model().db
 
         r = self.register
@@ -338,12 +395,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.default_author_link = DefaultAuthorLink(self.default_author_link_container)
         self.default_author_link.changed_signal.connect(self.changed_signal)
         r('gui_layout', config, restart_required=True, choices=[(_('Wide'), 'wide'), (_('Narrow'), 'narrow')])
-        r('ui_style', gprefs, restart_required=True, choices=[(_('System default'), 'system'), (_('calibre style'),
-                    'calibre')])
+        r('hidpi', gprefs, restart_required=True, choices=[(_('Automatic'), 'auto'), (_('On'), 'on'), (_('Off'), 'off')])
+        if isosx:
+            self.opt_hidpi.setVisible(False), self.label_hidpi.setVisible(False)
+        r('ui_style', gprefs, restart_required=True, choices=[(_('System default'), 'system'), (_('calibre style'), 'calibre')])
         r('book_list_tooltips', gprefs)
+        r('dnd_merge', gprefs)
+        r('wrap_toolbar_text', gprefs, restart_required=True)
+        r('show_layout_buttons', gprefs, restart_required=True)
         r('row_numbers_in_book_list', gprefs)
-        r('tag_browser_old_look', gprefs, restart_required=True)
+        r('tag_browser_old_look', gprefs)
         r('tag_browser_hide_empty_categories', gprefs)
+        r('tag_browser_show_tooltips', gprefs)
         r('bd_show_cover', gprefs)
         r('bd_overlay_cover_size', gprefs)
         r('cover_grid_width', gprefs)
@@ -353,6 +416,12 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('cover_grid_spacing', gprefs)
         r('cover_grid_show_title', gprefs)
         r('tag_browser_show_counts', gprefs)
+        r('tag_browser_item_padding', gprefs)
+
+        r('qv_respects_vls', gprefs)
+        r('qv_dclick_changes_column', gprefs)
+        r('qv_retkey_changes_column', gprefs)
+        r('qv_follows_column', gprefs)
 
         r('cover_flow_queue_length', config, restart_required=True)
         r('cover_browser_reflections', gprefs)
@@ -365,6 +434,13 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('emblem_position', gprefs, choices=[
             (_('Left'), 'left'), (_('Top'), 'top'), (_('Right'), 'right'), (_('Bottom'), 'bottom')])
         r('book_list_extra_row_spacing', gprefs)
+        r('booklist_grid', gprefs)
+        r('book_details_narrow_comments_layout', gprefs, choices=[(_('Float'), 'float'), (_('Columns'), 'columns')])
+        self.opt_book_details_narrow_comments_layout.setToolTip(textwrap.fill(_(
+            'Choose how the text is laid out when using the "Narrow" user interface layout.'
+            ' A value of "Float" means that the comments text will wrap around'
+            ' the other metadata fields, while a value of "Columns" means that'
+            ' the comments will be in a separate fixed width column.')))
         self.cover_browser_title_template_button.clicked.connect(self.edit_cb_title_template)
         self.id_links_button.clicked.connect(self.edit_id_link_rules)
 
@@ -380,7 +456,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                  if l != lang]
         if lang != 'en':
             items.append(('en', get_esc_lang('en')))
-        items.sort(cmp=lambda x, y: cmp(x[1].lower(), y[1].lower()))
+        items.sort(key=lambda x: x[1].lower())
         choices = [(y, x) for x, y in items]
         # Default language is the autodetected one
         choices = [(get_language(lang), lang)] + choices
@@ -410,14 +486,14 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('tags_browser_collapse_at', gprefs)
         r('tag_browser_dont_collapse', gprefs, setting=CommaSeparatedList)
 
-        choices = set([k for k in db.field_metadata.all_field_keys()
-                if (db.field_metadata[k]['is_category'] and
-                   (db.field_metadata[k]['datatype'] in ['text', 'series', 'enumeration']) and
-                    not db.field_metadata[k]['display'].get('is_names', False)) or
-                   (db.field_metadata[k]['datatype'] in ['composite'] and
-                    db.field_metadata[k]['display'].get('make_category', False))])
-        choices -= set(['authors', 'publisher', 'formats', 'news', 'identifiers'])
-        choices |= set(['search'])
+        choices = {k for k in db.field_metadata.all_field_keys()
+                if (db.field_metadata[k]['is_category'] and (
+                    db.field_metadata[k]['datatype'] in ['text', 'series', 'enumeration'
+                    ]) and not db.field_metadata[k]['display'].get('is_names', False)) or (
+                    db.field_metadata[k]['datatype'] in ['composite'
+                    ] and db.field_metadata[k]['display'].get('make_category', False))}
+        choices -= {'authors', 'publisher', 'formats', 'news', 'identifiers'}
+        choices |= {'search'}
         self.opt_categories_using_hierarchy.update_items_cache(choices)
         r('categories_using_hierarchy', db.prefs, setting=CommaSeparatedList,
           choices=sorted(list(choices), key=sort_key))
@@ -434,8 +510,19 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.field_display_order)
         self.display_model.dataChanged.connect(self.changed_signal)
         self.field_display_order.setModel(self.display_model)
-        self.df_up_button.clicked.connect(self.move_df_up)
-        self.df_down_button.clicked.connect(self.move_df_down)
+        connect_lambda(self.df_up_button.clicked, self,
+                lambda self: move_field_up(self.field_display_order, self.display_model))
+        connect_lambda(self.df_down_button.clicked, self,
+                lambda self: move_field_down(self.field_display_order, self.display_model))
+
+        self.qv_display_model = QVDisplayedFields(self.gui.current_db,
+                self.qv_display_order)
+        self.qv_display_model.dataChanged.connect(self.changed_signal)
+        self.qv_display_order.setModel(self.qv_display_model)
+        connect_lambda(self.qv_up_button.clicked, self,
+                lambda self: move_field_up(self.qv_display_order, self.qv_display_model))
+        connect_lambda(self.qv_down_button.clicked, self,
+                lambda self: move_field_down(self.qv_display_order, self.qv_display_model))
 
         self.edit_rules = EditRules(self.tabWidget)
         self.edit_rules.changed.connect(self.changed_signal)
@@ -455,8 +542,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.tabWidget.setCurrentIndex(0)
         keys = [QKeySequence('F11', QKeySequence.PortableText), QKeySequence(
             'Ctrl+Shift+F', QKeySequence.PortableText)]
-        keys = [unicode(x.toString(QKeySequence.NativeText)) for x in keys]
-        self.fs_help_msg.setText(unicode(self.fs_help_msg.text())%(
+        keys = [unicode_type(x.toString(QKeySequence.NativeText)) for x in keys]
+        self.fs_help_msg.setText(unicode_type(self.fs_help_msg.text())%(
             _(' or ').join(keys)))
         self.size_calculated.connect(self.update_cg_cache_size, type=Qt.QueuedConnection)
         self.tabWidget.currentChanged.connect(self.tab_changed)
@@ -478,8 +565,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         b.clicked.connect(self.restore_cover_grid_appearance)
         self.cover_grid_empty_cache.clicked.connect(self.empty_cache)
         self.cover_grid_open_cache.clicked.connect(self.open_cg_cache)
-        self.cover_grid_smaller_cover.clicked.connect(partial(self.resize_cover, True))
-        self.cover_grid_larger_cover.clicked.connect(partial(self.resize_cover, False))
+        connect_lambda(self.cover_grid_smaller_cover.clicked, self, lambda self: self.resize_cover(True))
+        connect_lambda(self.cover_grid_larger_cover.clicked, self, lambda self: self.resize_cover(False))
         self.cover_grid_reset_size.clicked.connect(self.cg_reset_size)
         self.opt_cover_grid_disk_cache_size.setMinimum(self.gui.grid_view.thumbnail_cache.min_disk_cache)
         self.opt_cover_grid_disk_cache_size.setMaximum(self.gui.grid_view.thumbnail_cache.min_disk_cache * 100)
@@ -544,6 +631,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.current_font = self.initial_font = font
         self.update_font_display()
         self.display_model.initialize()
+        self.qv_display_model.initialize()
         db = self.gui.current_db
         try:
             idx = self.gui.library_view.currentIndex().row()
@@ -597,6 +685,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.changed_signal.emit()
             self.update_font_display()
         self.display_model.restore_defaults()
+        self.qv_display_model.restore_defaults()
         self.edit_rules.clear()
         self.icon_rules.clear()
         self.grid_rules.clear()
@@ -639,36 +728,17 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def update_font_display(self):
         font = self.build_font_obj()
         fi = QFontInfo(font)
-        name = unicode(fi.family())
+        name = unicode_type(fi.family())
 
         self.font_display.setFont(font)
-        self.font_display.setText(name +
-                ' [%dpt]'%fi.pointSize())
-
-    def move_df_up(self):
-        idx = self.field_display_order.currentIndex()
-        if idx.isValid():
-            idx = self.display_model.move(idx, -1)
-            if idx is not None:
-                sm = self.field_display_order.selectionModel()
-                sm.select(idx, sm.ClearAndSelect)
-                self.field_display_order.setCurrentIndex(idx)
-
-    def move_df_down(self):
-        idx = self.field_display_order.currentIndex()
-        if idx.isValid():
-            idx = self.display_model.move(idx, 1)
-            if idx is not None:
-                sm = self.field_display_order.selectionModel()
-                sm.select(idx, sm.ClearAndSelect)
-                self.field_display_order.setCurrentIndex(idx)
+        self.font_display.setText(name + ' [%dpt]'%fi.pointSize())
 
     def change_font(self, *args):
         fd = QFontDialog(self.build_font_obj(), self)
         if fd.exec_() == fd.Accepted:
             font = fd.selectedFont()
             fi = QFontInfo(font)
-            self.current_font = [unicode(fi.family()), fi.pointSize(),
+            self.current_font = [unicode_type(fi.family()), fi.pointSize(),
                     fi.weight(), fi.italic(), font.stretch()]
             self.update_font_display()
             self.changed_signal.emit()
@@ -684,6 +754,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 QApplication.setFont(self.font_display.font())
                 rr = True
             self.display_model.commit()
+            self.qv_display_model.commit()
             self.edit_rules.commit(self.gui.current_db.prefs)
             self.icon_rules.commit(self.gui.current_db.prefs)
             self.grid_rules.commit(self.gui.current_db.prefs)
@@ -699,8 +770,10 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         m = gui.library_view.model()
         m.beginResetModel(), m.endResetModel()
         self.update_font_display()
+        gui.tags_view.set_look_and_feel()
         gui.tags_view.reread_collapse_parameters()
         gui.library_view.refresh_book_details()
+        gui.library_view.refresh_grid()
         gui.library_view.set_row_header_visibility()
         gui.cover_flow.setShowReflections(gprefs['cover_browser_reflections'])
         gui.cover_flow.setPreserveAspectRatio(gprefs['cb_preserve_aspect_ratio'])
@@ -710,6 +783,9 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             getattr(gui, view + '_view').set_row_header_visibility()
         gui.library_view.refresh_row_sizing()
         gui.grid_view.refresh_settings()
+        qv = get_quickview_action_plugin()
+        if qv:
+            qv.refill_quickview()
 
 
 if __name__ == '__main__':
