@@ -20,14 +20,15 @@ from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.saved_search_editor import SavedSearchEditor
 from calibre.gui2.dialogs.search import SearchDialog
 from calibre.utils.icu import primary_sort_key
+from polyglot.builtins import unicode_type, string_or_bytes
 
 QT_HIDDEN_CLEAR_ACTION = '_q_qlineeditclearaction'
 
 
-class AsYouType(unicode):
+class AsYouType(unicode_type):
 
     def __new__(cls, text):
-        self = unicode.__new__(cls, text)
+        self = unicode_type.__new__(cls, text)
         self.as_you_type = True
         return self
 
@@ -182,10 +183,10 @@ class SearchBox2(QComboBox):  # {{{
         self.setFocus(Qt.OtherFocusReason)
 
     def search_done(self, ok):
-        if isinstance(ok, basestring):
+        if isinstance(ok, string_or_bytes):
             self.setToolTip(ok)
             ok = False
-        if not unicode(self.currentText()).strip():
+        if not unicode_type(self.currentText()).strip():
             self.clear(emit_search=False)
             return
         self._in_a_search = ok
@@ -208,7 +209,7 @@ class SearchBox2(QComboBox):  # {{{
         if event.key() in (Qt.Key_Return, Qt.Key_Enter):
             self.do_search()
             self.focus_to_library.emit()
-        elif self.as_you_type and unicode(event.text()):
+        elif self.as_you_type and unicode_type(event.text()):
             self.timer.start(1500)
 
     # Comes from the combobox itself
@@ -240,7 +241,7 @@ class SearchBox2(QComboBox):  # {{{
 
     def _do_search(self, store_in_history=True, as_you_type=False):
         self.hide_completer_popup()
-        text = unicode(self.currentText()).strip()
+        text = unicode_type(self.currentText()).strip()
         if not text:
             return self.clear()
         if as_you_type:
@@ -258,7 +259,7 @@ class SearchBox2(QComboBox):  # {{{
                 self.insertItem(0, t)
             self.setCurrentIndex(0)
             self.block_signals(False)
-            history = [unicode(self.itemText(i)) for i in
+            history = [unicode_type(self.itemText(i)) for i in
                     range(self.count())]
             config[self.opt_name] = history
 
@@ -301,7 +302,7 @@ class SearchBox2(QComboBox):  # {{{
 
     @property
     def current_text(self):
-        return unicode(self.lineEdit().text())
+        return unicode_type(self.lineEdit().text())
 
     # }}}
 
@@ -365,7 +366,7 @@ class SavedSearchBox(QComboBox):  # {{{
     def saved_search_selected(self, qname):
         from calibre.gui2.ui import get_gui
         db = get_gui().current_db
-        qname = unicode(qname)
+        qname = unicode_type(qname)
         if qname is None or not qname.strip():
             self.search_box.clear()
             return
@@ -392,9 +393,9 @@ class SavedSearchBox(QComboBox):  # {{{
     def save_search_button_clicked(self):
         from calibre.gui2.ui import get_gui
         db = get_gui().current_db
-        name = unicode(self.currentText())
+        name = unicode_type(self.currentText())
         if not name.strip():
-            name = unicode(self.search_box.text()).replace('"', '')
+            name = unicode_type(self.search_box.text()).replace('"', '')
         name = name.replace('\\', '')
         if not name:
             error_dialog(self, _('Create saved search'),
@@ -406,7 +407,7 @@ class SavedSearchBox(QComboBox):  # {{{
                          _('There is no search to save'), show=True)
             return
         db.saved_search_delete(name)
-        db.saved_search_add(name, unicode(self.search_box.text()))
+        db.saved_search_add(name, unicode_type(self.search_box.text()))
         # now go through an initialization cycle to ensure that the combobox has
         # the new search in it, that it is selected, and that the search box
         # references the new search instead of the text in the search.
@@ -427,10 +428,10 @@ class SavedSearchBox(QComboBox):  # {{{
                        '<b>permanently deleted</b>. Are you sure?') +
                        '</p>', 'saved_search_delete', self):
             return
-        ss = db.saved_search_lookup(unicode(self.currentText()))
+        ss = db.saved_search_lookup(unicode_type(self.currentText()))
         if ss is None:
             return
-        db.saved_search_delete(unicode(self.currentText()))
+        db.saved_search_delete(unicode_type(self.currentText()))
         self.clear()
         self.search_box.clear()
         self.changed.emit()
@@ -442,7 +443,7 @@ class SavedSearchBox(QComboBox):  # {{{
         idx = self.currentIndex()
         if idx < 0:
             return
-        self.search_box.set_search_string(db.saved_search_lookup(unicode(self.currentText())))
+        self.search_box.set_search_string(db.saved_search_lookup(unicode_type(self.currentText())))
 
     # }}}
 
@@ -466,7 +467,7 @@ class SearchBoxMixin(object):  # {{{
         self.search.setMaximumWidth(self.width()-150)
         self.action_focus_search = QAction(self)
         shortcuts = list(
-                map(lambda x:unicode(x.toString(QKeySequence.PortableText)),
+                map(lambda x:unicode_type(x.toString(QKeySequence.PortableText)),
                 QKeySequence.keyBindings(QKeySequence.Find)))
         shortcuts += ['/', 'Alt+S']
         self.keyboard.register_shortcut('start search', _('Start search'),
@@ -474,7 +475,7 @@ class SearchBoxMixin(object):  # {{{
         self.action_focus_search.triggered.connect(self.focus_search_box)
         self.addAction(self.action_focus_search)
         self.search.setStatusTip(re.sub(r'<\w+>', ' ',
-            unicode(self.search.toolTip())))
+            unicode_type(self.search.toolTip())))
         self.set_highlight_only_button_icon()
         self.highlight_only_button.clicked.connect(self.highlight_only_clicked)
         tt = _('Enable or disable search highlighting.') + '<br><br>'
