@@ -19,6 +19,7 @@ from calibre.gui2.store.search.download_thread import DetailsThreadPool, \
     CoverThreadPool
 from calibre.utils.icu import sort_key
 from calibre.utils.search_query_parser import SearchQueryParser
+from polyglot.builtins import unicode_type
 
 
 def comparable_price(text):
@@ -153,15 +154,15 @@ class Matches(QAbstractItemModel):
         # Remove filter identifiers
         # Remove the prefix.
         for loc in ('all', 'author', 'author2', 'authors', 'title', 'title2'):
-            query = re.sub(r'%s:"(?P<a>[^\s"]+)"' % loc, '\g<a>', query)
+            query = re.sub(r'%s:"(?P<a>[^\s"]+)"' % loc, r'\g<a>', query)
             query = query.replace('%s:' % loc, '')
         # Remove the prefix and search text.
         for loc in ('cover', 'download', 'downloads', 'drm', 'format', 'formats', 'price', 'store'):
             query = re.sub(r'%s:"[^"]"' % loc, '', query)
             query = re.sub(r'%s:[^\s]*' % loc, '', query)
         # Remove whitespace
-        query = re.sub('\s', '', query)
-        mod_query = re.sub('\s', '', mod_query)
+        query = re.sub(r'\s', '', query)
+        mod_query = re.sub(r'\s', '', mod_query)
         # If mod_query and query are the same then there were no filter modifiers
         # so this isn't a filterable query.
         if mod_query == query:
@@ -231,7 +232,13 @@ class Matches(QAbstractItemModel):
             if col == 1:
                 return ('<p>%s</p>' % result.title)
             elif col == 2:
-                return ('<p>' + _('Detected price as: %s. Check with the store before making a purchase to verify this price is correct. This price often does not include promotions the store may be running.') % result.price + '</p>')  # noqa
+                if result.price:
+                    return ('<p>' + _(
+                        'Detected price as: %s. Check with the store before making a purchase'
+                        ' to verify this price is correct. This price often does not include'
+                        ' promotions the store may be running.') % result.price + '</p>')
+                return '<p>' + _(
+                    'No price was found')
             elif col == 3:
                 if result.drm == SearchResult.DRM_LOCKED:
                     return ('<p>' + _('This book as been detected as having DRM restrictions. This book may not work with your reader and you will have limitations placed upon you as to what you can do with this book. Check with the store before making any purchases to ensure you can actually read this book.') + '</p>')  # noqa
@@ -284,9 +291,9 @@ class Matches(QAbstractItemModel):
         if not self.matches:
             return
         descending = order == Qt.DescendingOrder
-        self.all_matches.sort(None,
-            lambda x: sort_key(unicode(self.data_as_text(x, col))),
-            descending)
+        self.all_matches.sort(
+            key=lambda x: sort_key(unicode_type(self.data_as_text(x, col))),
+            reverse=descending)
         self.reorder_matches()
         if reset:
             self.beginResetModel(), self.endResetModel()
@@ -386,7 +393,7 @@ class SearchFilter(SearchQueryParser):
         if location not in self.USABLE_LOCATIONS:
             return set([])
         matches = set([])
-        all_locs = set(self.USABLE_LOCATIONS) - set(['all'])
+        all_locs = set(self.USABLE_LOCATIONS) - {'all'}
         locations = all_locs if location == 'all' else [location]
         q = {
              'affiliate': attrgetter('affiliate'),
