@@ -12,7 +12,7 @@ from functools import partial
 from PyQt5.Qt import (
     QGridLayout, QSize, QListView, QStyledItemDelegate, QLabel, QPixmap,
     QApplication, QSizePolicy, QAbstractListModel, Qt, QRect, QCheckBox,
-    QPainter, QModelIndex, QSortFilterProxyModel, QLineEdit, QToolButton,
+    QPainter, QSortFilterProxyModel, QLineEdit, QToolButton,
     QIcon, QFormLayout, pyqtSignal, QTreeWidget, QTreeWidgetItem, QVBoxLayout,
     QMenu, QInputDialog, QHBoxLayout)
 
@@ -20,14 +20,16 @@ from calibre import fit_image
 from calibre.constants import plugins
 from calibre.ebooks.metadata import string_to_authors
 from calibre.ebooks.metadata.book.base import Metadata
-from calibre.gui2 import choose_files, error_dialog, pixmap_to_data
+from calibre.gui2 import choose_files, error_dialog, pixmap_to_data, empty_index
 from calibre.gui2.languages import LanguagesEdit
 from calibre.gui2.tweak_book import current_container, tprefs
 from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.tweak_book.file_list import name_is_ok
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.localization import get_lang, canonicalize_lang
-from calibre.utils.icu import sort_key
+from calibre.utils.icu import numeric_sort_key
+from polyglot.builtins import unicode_type
+
 
 class ChooseName(Dialog):  # {{{
 
@@ -58,30 +60,40 @@ class ChooseName(Dialog):  # {{{
         return False
 
     def verify(self):
-        return name_is_ok(unicode(self.name_edit.text()), self.show_error)
+        return name_is_ok(unicode_type(self.name_edit.text()), self.show_error)
 
     def accept(self):
         if not self.verify():
             return error_dialog(self, _('No name specified'), _(
                 'You must specify a file name for the new file, with an extension.'), show=True)
-        n = unicode(self.name_edit.text()).replace('\\', '/')
+        n = unicode_type(self.name_edit.text()).replace('\\', '/')
         name, ext = n.rpartition('.')[0::2]
         self.filename = name + '.' + ext.lower()
         super(ChooseName, self).accept()
 # }}}
 
 # Images {{{
+
+
 class ImageDelegate(QStyledItemDelegate):
 
     MARGIN = 4
 
     def __init__(self, parent):
         super(ImageDelegate, self).__init__(parent)
+        self.current_basic_size = tprefs.get('image-thumbnail-preview-size', [120, 160])
+        self.set_dimensions()
+
+    def change_size(self, increase=True):
+        percent = 10 if increase else -10
+        frac = (100 + percent) / 100.
+        self.current_basic_size[0] = min(1200, max(40, int(frac * self.current_basic_size[0])))
+        self.current_basic_size[1] = min(1600, max(60, int(frac * self.current_basic_size[1])))
+        tprefs.set('image-thumbnail-preview-size', self.current_basic_size)
         self.set_dimensions()
-        self.cover_cache = {}
 
     def set_dimensions(self):
-        width, height = 120, 160
+        width, height = self.current_basic_size
         self.cover_size = QSize(width, height)
         f = self.parent().font()
         sz = f.pixelSize()
@@ -90,6 +102,7 @@ class ImageDelegate(QStyledItemDelegate):
         self.title_height = max(25, sz + 10)
         self.item_size = self.cover_size + QSize(2 * self.MARGIN, (2 * self.MARGIN) + self.title_height)
         self.calculate_spacing()
+        self.cover_cache = {}
 
     def calculate_spacing(self):
         self.spacing = max(10, min(50, int(0.1 * self.item_size.width())))
@@ -98,8 +111,8 @@ class ImageDelegate(QStyledItemDelegate):
         return self.item_size
 
     def paint(self, painter, option, index):
-        QStyledItemDelegate.paint(self, painter, option, QModelIndex())  # draw the hover and selection highlights
-        name = unicode(index.data(Qt.DisplayRole) or '')
+        QStyledItemDelegate.paint(self, painter, option, empty_index)  # draw the hover and selection highlights
+        name = unicode_type(index.data(Qt.DisplayRole) or '')
         cover = self.cover_cache.get(name, None)
         if cover is None:
             cover = self.cover_cache[name] = QPixmap()
@@ -108,11 +121,16 @@ class ImageDelegate(QStyledItemDelegate):
             except:
                 pass
             else:
+                try:
+                    dpr = painter.device().devicePixelRatioF()
+                except AttributeError:
+                    dpr = painter.device().devicePixelRatio()
                 cover.loadFromData(raw)
+                cover.setDevicePixelRatio(dpr)
                 if not cover.isNull():
                     scaled, width, height = fit_image(cover.width(), cover.height(), self.cover_size.width(), self.cover_size.height())
                     if scaled:
-                        cover = self.cover_cache[name] = cover.scaled(width, height, transformMode=Qt.SmoothTransformation)
+                        cover = self.cover_cache[name] = cover.scaled(int(dpr*width), int(dpr*height), transformMode=Qt.SmoothTransformation)
 
         painter.save()
         try:
@@ -121,8 +139,8 @@ class ImageDelegate(QStyledItemDelegate):
             trect = QRect(rect)
             rect.setBottom(rect.bottom() - self.title_height)
             if not cover.isNull():
-                dx = max(0, int((rect.width() - cover.width())/2.0))
-                dy = max(0, rect.height() - cover.height())
+                dx = max(0, int((rect.width() - int(cover.width()/cover.devicePixelRatio()))/2.0))
+                dy = max(0, rect.height() - int(cover.height()/cover.devicePixelRatio()))
                 rect.adjust(dx, dy, -dx, 0)
                 painter.drawPixmap(rect, cover)
             rect = trect
@@ -134,6 +152,7 @@ class ImageDelegate(QStyledItemDelegate):
         finally:
             painter.restore()
 
+
 class Images(QAbstractListModel):
 
     def __init__(self, parent):
@@ -146,7 +165,7 @@ class Images(QAbstractListModel):
         self.image_names = []
         self.image_cache = {}
         if c is not None:
-            for name in sorted(c.mime_map, key=sort_key):
+            for name in sorted(c.mime_map, key=numeric_sort_key):
                 if c.mime_map[name].startswith('image/'):
                     self.image_names.append(name)
 
@@ -170,6 +189,7 @@ class Images(QAbstractListModel):
             return name
         return None
 
+
 class InsertImage(Dialog):
 
     image_activated = pyqtSignal(object)
@@ -249,20 +269,33 @@ class InsertImage(Dialog):
             self.preserve_aspect_ratio = a = QCheckBox(_('Preserve aspect ratio'))
             a.setToolTip(_('Preserve the aspect ratio of the inserted image when rendering it full paged'))
             a.setChecked(tprefs['preserve_aspect_ratio_when_inserting_image'])
-            f.toggled.connect(lambda : (tprefs.set('insert_full_screen_image', f.isChecked()), a.setVisible(f.isChecked())))
-            a.toggled.connect(lambda : tprefs.set('preserve_aspect_ratio_when_inserting_image', a.isChecked()))
+            f.toggled.connect(self.full_page_image_toggled)
+            a.toggled.connect(self.par_toggled)
             a.setVisible(f.isChecked())
             h = QHBoxLayout()
             l.addLayout(h, 3, 0, 1, -1)
             h.addWidget(f), h.addStretch(10), h.addWidget(a)
+        b = self.bb.addButton(_('&Zoom in'), self.bb.ActionRole)
+        b.clicked.connect(self.zoom_in)
+        b.setIcon(QIcon(I('plus.png')))
+        b = self.bb.addButton(_('Zoom &out'), self.bb.ActionRole)
+        b.clicked.connect(self.zoom_out)
+        b.setIcon(QIcon(I('minus.png')))
         l.addWidget(self.bb, 4, 0, 1, 2)
 
+    def full_page_image_toggled(self):
+        tprefs.set('insert_full_screen_image', self.fullpage.isChecked())
+        self.preserve_aspect_ratio.setVisible(self.fullpage.isChecked())
+
+    def par_toggled(self):
+        tprefs.set('preserve_aspect_ratio_when_inserting_image', self.preserve_aspect_ratio.isChecked())
+
     def refresh(self):
         self.d.cover_cache.clear()
         self.model.refresh()
 
     def import_image(self):
-        path = choose_files(self, 'tweak-book-choose-image-for-import', _('Choose Image'),
+        path = choose_files(self, 'tweak-book-choose-image-for-import', _('Choose image'),
                             filters=[(_('Images'), ('jpg', 'jpeg', 'png', 'gif', 'svg'))], all_files=True, select_only_single_file=True)
         if path:
             path = path[0]
@@ -274,6 +307,14 @@ class InsertImage(Dialog):
                 self.accept()
                 self.chosen_image_is_external = (d.filename, path)
 
+    def zoom_in(self):
+        self.d.change_size(increase=True)
+        self.model.beginResetModel(), self.model.endResetModel()
+
+    def zoom_out(self):
+        self.d.change_size(increase=False)
+        self.model.beginResetModel(), self.model.endResetModel()
+
     def paste_image(self):
         c = QApplication.instance().clipboard()
         img = c.image()
@@ -300,25 +341,27 @@ class InsertImage(Dialog):
 
     def activated(self, index):
         if self.for_browsing:
-            return self.image_activated.emit(unicode(index.data() or ''))
+            return self.image_activated.emit(unicode_type(index.data() or ''))
         self.chosen_image_is_external = False
         self.accept()
 
     def accept(self):
-        self.chosen_image = unicode(self.view.currentIndex().data() or '')
+        self.chosen_image = unicode_type(self.view.currentIndex().data() or '')
         super(InsertImage, self).accept()
 
     def filter_changed(self, *args):
-        f = unicode(self.filter.text())
+        f = unicode_type(self.filter.text())
         self.fm.setFilterFixedString(f)
 # }}}
 
+
 def get_resource_data(rtype, parent):
     if rtype == 'image':
         d = InsertImage(parent)
         if d.exec_() == d.Accepted:
             return d.chosen_image, d.chosen_image_is_external, d.fullpage.isChecked(), d.preserve_aspect_ratio.isChecked()
 
+
 def create_folder_tree(container):
     root = {}
 
@@ -331,6 +374,7 @@ def create_folder_tree(container):
             current[x] = current = current.get(x, {})
     return root
 
+
 class ChooseFolder(Dialog):  # {{{
 
     def __init__(self, msg=None, parent=None):
@@ -356,7 +400,7 @@ class ChooseFolder(Dialog):  # {{{
 
         def process(node, parent):
             parent.setIcon(0, QIcon(I('mimetypes/dir.png')))
-            for child in sorted(node, key=sort_key):
+            for child in sorted(node, key=numeric_sort_key):
                 c = QTreeWidgetItem(parent, (child,))
                 process(node[child], c)
         process(create_folder_tree(current_container()), self.root)
@@ -375,8 +419,8 @@ class ChooseFolder(Dialog):  # {{{
 
     def create_folder(self, item):
         text, ok = QInputDialog.getText(self, _('Folder name'), _('Enter a name for the new folder'))
-        if ok and unicode(text):
-            c = QTreeWidgetItem(item, (unicode(text),))
+        if ok and unicode_type(text):
+            c = QTreeWidgetItem(item, (unicode_type(text),))
             c.setIcon(0, QIcon(I('mimetypes/dir.png')))
             for item in self.folders.selectedItems():
                 item.setSelected(False)
@@ -386,7 +430,7 @@ class ChooseFolder(Dialog):  # {{{
     def folder_path(self, item):
         ans = []
         while item is not self.root:
-            ans.append(unicode(item.text(0)))
+            ans.append(unicode_type(item.text(0)))
             item = item.parent()
         return tuple(reversed(ans))
 
@@ -426,34 +470,35 @@ class NewBook(Dialog):  # {{{
         bb.clear()
         bb.addButton(bb.Cancel)
         b = bb.addButton('&EPUB', bb.AcceptRole)
-        b.clicked.connect(partial(self.set_fmt, 'epub'))
+        connect_lambda(b.clicked, self, lambda self: self.set_fmt('epub'))
         b = bb.addButton('&AZW3', bb.AcceptRole)
-        b.clicked.connect(partial(self.set_fmt, 'azw3'))
+        connect_lambda(b.clicked, self, lambda self: self.set_fmt('azw3'))
 
     def set_fmt(self, fmt):
         self.fmt = fmt
 
     def accept(self):
-        tprefs.set('previous_new_book_authors', unicode(self.authors.text()))
-        tprefs.set('previous_new_book_lang', (self.languages.lang_codes or [get_lang()])[0])
+        with tprefs:
+            tprefs.set('previous_new_book_authors', unicode_type(self.authors.text()))
+            tprefs.set('previous_new_book_lang', (self.languages.lang_codes or [get_lang()])[0])
+            self.languages.update_recently_used()
         super(NewBook, self).accept()
 
     @property
     def mi(self):
-        mi = Metadata(unicode(self.title.text()).strip() or _('Unknown'))
-        mi.authors = string_to_authors(unicode(self.authors.text()).strip()) or [_('Unknown')]
+        mi = Metadata(unicode_type(self.title.text()).strip() or _('Unknown'))
+        mi.authors = string_to_authors(unicode_type(self.authors.text()).strip()) or [_('Unknown')]
         mi.languages = self.languages.lang_codes or [get_lang()]
         return mi
 
 # }}}
 
+
 if __name__ == '__main__':
     app = QApplication([])  # noqa
     from calibre.gui2.tweak_book import set_current_container
     from calibre.gui2.tweak_book.boss import get_container
     set_current_container(get_container(sys.argv[-1]))
 
-    d = ChooseFolder()
-    if d.exec_() == d.Accepted:
-        print (repr(d.chosen_folder))
-
+    d = InsertImage(for_browsing=True)
+    d.exec_()
