@@ -24,21 +24,26 @@ from calibre.gui2.tweak_book.widgets import Dialog, PlainTextEdit
 from calibre.utils.config import JSONConfig
 from calibre.utils.icu import string_length as strlen
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import codepoint_to_chr, iteritems, itervalues, unicode_type, range
 
-string_length = lambda x: strlen(unicode(x))  # Needed on narrow python builds, as subclasses of unicode dont work
+string_length = lambda x: strlen(unicode_type(x))  # Needed on narrow python builds, as subclasses of unicode dont work
 KEY = Qt.Key_J
 MODIFIER = Qt.META if isosx else Qt.CTRL
 
 SnipKey = namedtuple('SnipKey', 'trigger syntaxes')
+
+
 def snip_key(trigger, *syntaxes):
     if '*' in syntaxes:
         syntaxes = all_text_syntaxes
     return SnipKey(trigger, frozenset(syntaxes))
 
+
 def contains(l1, r1, l2, r2):
     # True iff (l2, r2) if contained in (l1, r1)
     return l2 > l1 and r2 < r1
 
+
 builtin_snippets = {  # {{{
     snip_key('Lorem', 'html', 'xml'):  {
         'description': _('Insert filler text'),
@@ -85,18 +90,21 @@ obtain some advantage from it? But.</p>
 
 # Parsing of snippets {{{
 escape = unescape = None
+
+
 def escape_funcs():
     global escape, unescape
     if escape is None:
-        escapem = {('\\' + x):unichr(i+1) for i, x in enumerate('\\${}')}
+        escapem = {('\\' + x):codepoint_to_chr(i+1) for i, x in enumerate('\\${}')}
         escape_pat = re.compile('|'.join(map(re.escape, escapem)))
         escape = lambda x: escape_pat.sub(lambda m: escapem[m.group()], x.replace(r'\\', '\x01'))
-        unescapem = {v:k[1] for k, v in escapem.iteritems()}
+        unescapem = {v:k[1] for k, v in iteritems(escapem)}
         unescape_pat = re.compile('|'.join(unescapem))
         unescape = lambda x:unescape_pat.sub(lambda m:unescapem[m.group()], x)
     return escape, unescape
 
-class TabStop(unicode):
+
+class TabStop(unicode_type):
 
     def __new__(self, raw, start_offset, tab_stops, is_toplevel=True):
         if raw.endswith('}'):
@@ -107,7 +115,7 @@ class TabStop(unicode):
             for c in child_stops:
                 c.parent = self
             tab_stops.extend(child_stops)
-            self = unicode.__new__(self, uraw)
+            self = unicode_type.__new__(self, uraw)
             if num.endswith('*'):
                 self.takes_selection = True
                 num = num[:-1]
@@ -115,7 +123,7 @@ class TabStop(unicode):
                 self.takes_selection = False
             self.num = int(num)
         else:
-            self = unicode.__new__(self, '')
+            self = unicode_type.__new__(self, '')
             self.num = int(raw[1:])
             self.takes_selection = False
         self.start = start_offset
@@ -127,7 +135,8 @@ class TabStop(unicode):
 
     def __repr__(self):
         return 'TabStop(text=%s num=%d start=%d is_mirror=%s takes_selection=%s is_toplevel=%s)' % (
-            unicode.__repr__(self), self.num, self.start, self.is_mirror, self.takes_selection, self.is_toplevel)
+            unicode_type.__repr__(self), self.num, self.start, self.is_mirror, self.takes_selection, self.is_toplevel)
+
 
 def parse_template(template, start_offset=0, is_toplevel=True, grouped=True):
     escape, unescape = escape_funcs()
@@ -155,9 +164,11 @@ def parse_template(template, start_offset=0, is_toplevel=True, grouped=True):
 
 # }}}
 
+
 _snippets = None
 user_snippets = JSONConfig('editor_snippets')
 
+
 def snippets(refresh=False):
     global _snippets
     if _snippets is None or refresh:
@@ -166,11 +177,12 @@ def snippets(refresh=False):
             if snip['trigger'] and isinstance(snip['trigger'], type('')):
                 key = snip_key(snip['trigger'], *snip['syntaxes'])
                 _snippets[key] = {'template':snip['template'], 'description':snip['description']}
-        _snippets = sorted(_snippets.iteritems(), key=(lambda (key, snip):string_length(key.trigger)), reverse=True)
+        _snippets = sorted(iteritems(_snippets), key=(lambda key_snip:string_length(key_snip[0].trigger)), reverse=True)
     return _snippets
 
 # Editor integration {{{
 
+
 class EditorTabStop(object):
 
     def __init__(self, left, tab_stops, editor):
@@ -218,6 +230,7 @@ class EditorTabStop(object):
             c = editor.textCursor()
             c.setPosition(self.left), c.setPosition(self.right, c.KeepAnchor)
             return editor.selected_text_from_cursor(c)
+
         def fset(self, text):
             editor = self.editor()
             if editor is None or self.is_deleted:
@@ -262,6 +275,7 @@ class EditorTabStop(object):
             if position <= self.right:
                 self.right += chars_added
 
+
 class Template(list):
 
     def __new__(self, tab_stops):
@@ -329,6 +343,7 @@ class Template(list):
                 dist, ans = x, c
         return ans
 
+
 def expand_template(editor, trigger, template):
     c = editor.textCursor()
     c.beginEditBlock()
@@ -337,7 +352,7 @@ def expand_template(editor, trigger, template):
     left = right - string_length(trigger)
     text, tab_stops = parse_template(template)
     c.setPosition(left), c.setPosition(right, c.KeepAnchor), c.insertText(text)
-    editor_tab_stops = [EditorTabStop(left, ts, editor) for ts in tab_stops.itervalues()]
+    editor_tab_stops = [EditorTabStop(left, ts, editor) for ts in itervalues(tab_stops)]
 
     tl = Template(editor_tab_stops)
     if tl.has_tab_stops:
@@ -348,6 +363,7 @@ def expand_template(editor, trigger, template):
     c.endEditBlock()
     return tl
 
+
 def find_matching_snip(text, syntax=None, snip_func=None):
     ans_snip = ans_trigger = None
     for key, snip in (snip_func or snippets)():
@@ -356,6 +372,7 @@ def find_matching_snip(text, syntax=None, snip_func=None):
             break
     return ans_snip, ans_trigger
 
+
 class SnippetManager(QObject):
 
     def __init__(self, editor):
@@ -405,6 +422,7 @@ class SnippetManager(QObject):
             if snip is None:
                 error_dialog(self.parent(), _('No snippet found'), _(
                     'No matching snippet was found'), show=True)
+                self.last_selected_text = self.last_selected_text or lst
                 return True
             template = expand_template(editor, trigger, snip['template'])
             if template.has_tab_stops:
@@ -420,6 +438,7 @@ class SnippetManager(QObject):
 
 # Config {{{
 
+
 class SnippetTextEdit(PlainTextEdit):
 
     def __init__(self, text, parent=None):
@@ -433,6 +452,7 @@ class SnippetTextEdit(PlainTextEdit):
             return
         PlainTextEdit.keyPressEvent(self, ev)
 
+
 class EditSnippet(QWidget):
 
     def __init__(self, parent=None):
@@ -509,7 +529,7 @@ class EditSnippet(QWidget):
         self.template.setPlainText(snip.get('template') or '')
 
         ftypes = snip.get('syntaxes', ())
-        for i in xrange(self.types.count()):
+        for i in range(self.types.count()):
             i = self.types.item(i)
             ftype = i.data(Qt.UserRole)
             i.setCheckState(Qt.Checked if ftype in ftypes else Qt.Unchecked)
@@ -521,9 +541,10 @@ class EditSnippet(QWidget):
     def snip(self):
         def fset(self, snip):
             self.apply_snip(snip)
+
         def fget(self):
             ftypes = []
-            for i in xrange(self.types.count()):
+            for i in range(self.types.count()):
                 i = self.types.item(i)
                 if i.checkState() == Qt.Checked:
                     ftypes.append(i.data(Qt.UserRole))
@@ -544,10 +565,11 @@ class EditSnippet(QWidget):
             err = _('You must specify at least one file type')
         return err
 
+
 class UserSnippets(Dialog):
 
     def __init__(self, parent=None):
-        Dialog.__init__(self, _('Create/Edit snippets'), 'snippet-editor', parent=parent)
+        Dialog.__init__(self, _('Create/edit snippets'), 'snippet-editor', parent=parent)
         self.setWindowIcon(QIcon(I('snippets.png')))
 
     def setup_ui(self):
@@ -635,7 +657,7 @@ class UserSnippets(Dialog):
             else:
                 error_dialog(self, _('Invalid snippet'), err, show=True)
             return
-        user_snippets['snippets'] = [self.snip_list.item(i).data(Qt.UserRole) for i in xrange(self.snip_list.count())]
+        user_snippets['snippets'] = [self.snip_list.item(i).data(Qt.UserRole) for i in range(self.snip_list.count())]
         snippets(refresh=True)
         return Dialog.accept(self)
 
@@ -677,7 +699,7 @@ class UserSnippets(Dialog):
     def change_builtin(self):
         d = QDialog(self)
         lw = QListWidget(d)
-        for (trigger, syntaxes), snip in builtin_snippets.iteritems():
+        for (trigger, syntaxes), snip in iteritems(builtin_snippets):
             snip = copy.deepcopy(snip)
             snip['trigger'], snip['syntaxes'] = trigger, syntaxes
             i = QListWidgetItem(self.snip_to_text(snip), lw)
@@ -694,6 +716,7 @@ class UserSnippets(Dialog):
             self.edit_snip.apply_snip(lw.currentItem().data(Qt.UserRole), creating_snippet=True)
 # }}}
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
