@@ -1,37 +1,41 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+# License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-__license__ = 'GPL v3'
-__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
-
-import json, copy
+import copy
+import json
+from collections import OrderedDict, Counter
 from functools import partial
-from collections import OrderedDict
 
 from PyQt5.Qt import (
-    QWidget, QToolBar, Qt, QHBoxLayout, QSize, QIcon, QGridLayout, QLabel, QTimer,
-    QPushButton, pyqtSignal, QComboBox, QCheckBox, QSizePolicy, QVBoxLayout, QFont,
-    QLineEdit, QToolButton, QListView, QFrame, QApplication, QStyledItemDelegate,
-    QAbstractListModel, QModelIndex, QMenu, QItemSelection, QStackedLayout, QScrollArea)
+    QAbstractListModel, QApplication, QCheckBox, QComboBox, QFont, QFrame,
+    QGridLayout, QHBoxLayout, QIcon, QItemSelection, QLabel, QLineEdit, QListView,
+    QMenu, QMimeData, QModelIndex, QPushButton, QScrollArea, QSize, QSizePolicy,
+    QStackedLayout, QStyledItemDelegate, Qt, QTimer, QToolBar, QToolButton,
+    QVBoxLayout, QWidget, pyqtSignal, QAction, QKeySequence
+)
 
 import regex
-
 from calibre import prepare_string_for_xml
-from calibre.gui2 import error_dialog, info_dialog, choose_files, choose_save_file
+from calibre.gui2 import choose_files, choose_save_file, error_dialog, info_dialog
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.message_box import MessageBox
-from calibre.gui2.widgets2 import HistoryComboBox, FlowLayout
-from calibre.gui2.tweak_book import tprefs, editors, current_container
+from calibre.gui2.tweak_book import current_container, editors, tprefs
+from calibre.gui2.tweak_book.editor.snippets import (
+    KEY, MODIFIER, SnippetTextEdit, find_matching_snip, parse_template,
+    string_length
+)
 from calibre.gui2.tweak_book.function_replace import (
-    FunctionBox, functions as replace_functions, FunctionEditor, remove_function, Function)
+    Function, FunctionBox, FunctionEditor, functions as replace_functions,
+    remove_function
+)
 from calibre.gui2.tweak_book.widgets import BusyCursor
-from calibre.gui2.tweak_book.editor.snippets import find_matching_snip, parse_template, string_length, SnippetTextEdit, MODIFIER, KEY
-
+from calibre.gui2.widgets2 import FlowLayout, HistoryComboBox
 from calibre.utils.icu import primary_contains
+from calibre.ebooks.conversion.search_replace import REGEX_FLAGS, compile_regular_expression
+from polyglot.builtins import iteritems, unicode_type, range, error_message, filter
 
-REGEX_FLAGS = regex.VERSION1 | regex.WORD | regex.FULLCASE | regex.MULTILINE | regex.UNICODE
 
 # The search panel {{{
 
@@ -59,7 +63,7 @@ class PushButton(AnimatablePushButton):
 
     def __init__(self, text, action, parent):
         AnimatablePushButton.__init__(self, text, parent)
-        self.clicked.connect(lambda : parent.search_triggered.emit(action))
+        connect_lambda(self.clicked, parent, lambda parent: parent.search_triggered.emit(action))
 
 
 def expand_template(line_edit):
@@ -91,6 +95,7 @@ class HistoryBox(HistoryComboBox):
         self.disable_popup = tprefs['disable_completion_popup_for_search']
         self.clear_msg = clear_msg
         self.ignore_snip_expansion = False
+        self.lineEdit().setClearButtonEnabled(True)
 
     def event(self, ev):
         if ev.type() in (ev.ShortcutOverride, ev.KeyPress) and ev.key() == KEY and ev.modifiers() & MODIFIER:
@@ -134,7 +139,7 @@ class WhereBox(QComboBox):
             <dt><b>All style files</b></dt>
             <dd>Search in all style (CSS) files</dd>
             <dt><b>Selected files</b></dt>
-            <dd>Search in the files currently selected in the Files Browser</dd>
+            <dd>Search in the files currently selected in the File browser</dd>
             <dt><b>Open files</b></dt>
             <dd>Search in the files currently open in the editor</dd>
             <dt><b>Marked text</b></dt>
@@ -155,7 +160,7 @@ class WhereBox(QComboBox):
             return wm[self.currentIndex()]
 
         def fset(self, val):
-            self.setCurrentIndex({v:k for k, v in wm.iteritems()}[val])
+            self.setCurrentIndex({v:k for k, v in iteritems(wm)}[val])
         return property(fget=fget, fset=fset)
 
     def showPopup(self):
@@ -199,7 +204,7 @@ class ModeBox(QComboBox):
 
     def __init__(self, parent):
         QComboBox.__init__(self, parent)
-        self.addItems([_('Normal'), _('Regex'), _('Regex-Function')])
+        self.addItems([_('Normal'), _('Regex'), _('Regex-function')])
         self.setToolTip('<style>dd {margin-bottom: 1.5ex}</style>' + _(
             '''Select how the search expression is interpreted
             <dl>
@@ -207,8 +212,8 @@ class ModeBox(QComboBox):
             <dd>The search expression is treated as normal text, calibre will look for the exact text.</dd>
             <dt><b>Regex</b></dt>
             <dd>The search expression is interpreted as a regular expression. See the User Manual for more help on using regular expressions.</dd>
-            <dt><b>Regex-Function</b></dt>
-            <dd>The search expression is interpreted as a regular expression. The replace expression is an arbitrarily powerful python function.</dd>
+            <dt><b>Regex-function</b></dt>
+            <dd>The search expression is interpreted as a regular expression. The replace expression is an arbitrarily powerful Python function.</dd>
             </dl>'''))
 
     @dynamic_property
@@ -239,15 +244,16 @@ class SearchWidget(QWidget):
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         self.l = l = QGridLayout(self)
-        l.setContentsMargins(0, 0, 0, 0)
+        left, top, right, bottom = l.getContentsMargins()
+        l.setContentsMargins(0, 0, right, 0)
 
         self.fl = fl = QLabel(_('&Find:'))
         fl.setAlignment(Qt.AlignRight | Qt.AlignCenter)
-        self.find_text = ft = HistoryBox(self, _('Clear search history'))
+        self.find_text = ft = HistoryBox(self, _('Clear search &history'))
         ft.save_search.connect(self.save_search)
         ft.show_saved_searches.connect(self.show_saved_searches)
         ft.initialize('tweak_book_find_edit')
-        ft.lineEdit().returnPressed.connect(lambda : self.search_triggered.emit('find'))
+        connect_lambda(ft.lineEdit().returnPressed, self, lambda self: self.search_triggered.emit('find'))
         fl.setBuddy(ft)
         l.addWidget(fl, 0, 0)
         l.addWidget(ft, 0, 1)
@@ -255,7 +261,7 @@ class SearchWidget(QWidget):
 
         self.rl = rl = QLabel(_('&Replace:'))
         rl.setAlignment(Qt.AlignRight | Qt.AlignCenter)
-        self.replace_text = rt = HistoryBox(self, _('Clear replace history'))
+        self.replace_text = rt = HistoryBox(self, _('Clear replace &history'))
         rt.save_search.connect(self.save_search)
         rt.show_saved_searches.connect(self.show_saved_searches)
         rt.initialize('tweak_book_replace_edit')
@@ -360,7 +366,7 @@ class SearchWidget(QWidget):
     @dynamic_property
     def find(self):
         def fget(self):
-            return unicode(self.find_text.text())
+            return unicode_type(self.find_text.text())
 
         def fset(self, val):
             self.find_text.setText(val)
@@ -371,7 +377,7 @@ class SearchWidget(QWidget):
         def fget(self):
             if self.mode == 'function':
                 return self.functions.text()
-            return unicode(self.replace_text.text())
+            return unicode_type(self.replace_text.text())
 
         def fset(self, val):
             self.replace_text.setText(val)
@@ -443,16 +449,13 @@ class SearchWidget(QWidget):
 
     def pre_fill(self, text):
         if self.mode in ('regex', 'function'):
-            text = regex.escape(text, special_only=True)
+            text = regex.escape(text, special_only=True, literal_spaces=True)
         self.find = text
         self.find_text.lineEdit().setSelection(0, len(text)+10)
 
 # }}}
 
 
-regex_cache = {}
-
-
 class SearchPanel(QWidget):  # {{{
 
     search_triggered = pyqtSignal(object)
@@ -533,11 +536,67 @@ class SearchesModel(QAbstractListModel):
     def __init__(self, parent):
         QAbstractListModel.__init__(self, parent)
         self.searches = tprefs['saved_searches']
-        self.filtered_searches = list(xrange(len(self.searches)))
+        self.filtered_searches = list(range(len(self.searches)))
 
     def rowCount(self, parent=QModelIndex()):
         return len(self.filtered_searches)
 
+    def supportedDropActions(self):
+        return Qt.MoveAction
+
+    def flags(self, index):
+        ans = QAbstractListModel.flags(self, index)
+        if index.isValid():
+            ans |= Qt.ItemIsDragEnabled
+        else:
+            ans |= Qt.ItemIsDropEnabled
+        return ans
+
+    def mimeTypes(self):
+        return ['x-calibre/searches-rows', 'application/vnd.text.list']
+
+    def mimeData(self, indices):
+        ans = QMimeData()
+        names, rows = [], []
+        for i in indices:
+            if i.isValid():
+                names.append(i.data())
+                rows.append(i.row())
+        ans.setData('x-calibre/searches-rows', ','.join(map(str, rows)).encode('ascii'))
+        ans.setData('application/vnd.text.list', '\n'.join(names).encode('utf-8'))
+        return ans
+
+    def dropMimeData(self, data, action, row, column, parent):
+        if parent.isValid() or action != Qt.MoveAction or not data.hasFormat('x-calibre/searches-rows') or not self.filtered_searches:
+            return False
+        rows = sorted(map(int, bytes(bytearray(data.data('x-calibre/searches-rows'))).decode('ascii').split(',')))
+        moved_searches = [self.searches[self.filtered_searches[r]] for r in rows]
+        moved_searches_q = {id(s) for s in moved_searches}
+        insert_at = max(0, min(row, len(self.filtered_searches)))
+        while insert_at < len(self.filtered_searches):
+            s = self.searches[self.filtered_searches[insert_at]]
+            if id(s) in moved_searches_q:
+                insert_at += 1
+            else:
+                break
+        insert_before = id(self.searches[self.filtered_searches[insert_at]]) if insert_at < len(self.filtered_searches) else None
+        visible_searches = {id(self.searches[self.filtered_searches[r]]) for r in self.filtered_searches}
+        unmoved_searches = list(filter(lambda s:id(s) not in moved_searches_q, self.searches))
+        if insert_before is None:
+            searches = unmoved_searches + moved_searches
+        else:
+            idx = {id(x):i for i, x in enumerate(unmoved_searches)}[insert_before]
+            searches = unmoved_searches[:idx] + moved_searches + unmoved_searches[idx:]
+        filtered_searches = []
+        for i, s in enumerate(searches):
+            if id(s) in visible_searches:
+                filtered_searches.append(i)
+        self.modelAboutToBeReset.emit()
+        self.searches, self.filtered_searches = searches, filtered_searches
+        self.modelReset.emit()
+        tprefs['saved_searches'] = self.searches
+        return True
+
     def data(self, index, role):
         try:
             if role == Qt.DisplayRole:
@@ -555,7 +614,7 @@ class SearchesModel(QAbstractListModel):
         return None
 
     def do_filter(self, text):
-        text = unicode(text)
+        text = unicode_type(text)
         self.beginResetModel()
         self.filtered_searches = []
         for i, search in enumerate(self.searches):
@@ -563,6 +622,18 @@ class SearchesModel(QAbstractListModel):
                 self.filtered_searches.append(i)
         self.endResetModel()
 
+    def search_for_index(self, index):
+        try:
+            return self.searches[self.filtered_searches[index.row()]]
+        except IndexError:
+            pass
+
+    def index_for_search(self, search):
+        for row, si in enumerate(self.filtered_searches):
+            if self.searches[si] is search:
+                return self.index(row)
+        return self.index(-1)
+
     def move_entry(self, row, delta):
         a, b = row, row + delta
         if 0 <= b < len(self.filtered_searches):
@@ -575,7 +646,7 @@ class SearchesModel(QAbstractListModel):
     def add_searches(self, count=1):
         self.beginResetModel()
         self.searches = tprefs['saved_searches']
-        self.filtered_searches.extend(xrange(len(self.searches) - 1, len(self.searches) - 1 - count, -1))
+        self.filtered_searches.extend(range(len(self.searches) - count, len(self.searches), 1))
         self.endResetModel()
 
     def remove_searches(self, rows):
@@ -726,7 +797,7 @@ class EditSearch(QFrame):  # {{{
     @property
     def current_search(self):
         search = self.search.copy()
-        f = unicode(self.find.toPlainText())
+        f = unicode_type(self.find.toPlainText())
         search['find'] = f
         search['dot_all'] = bool(self.dot_all.isChecked())
         search['case_sensitive'] = bool(self.case_sensitive.isChecked())
@@ -734,7 +805,7 @@ class EditSearch(QFrame):  # {{{
         if search['mode'] == 'function':
             r = self.function.text()
         else:
-            r = unicode(self.replace.toPlainText())
+            r = unicode_type(self.replace.toPlainText())
         search['replace'] = r
         return search
 
@@ -753,7 +824,7 @@ class EditSearch(QFrame):  # {{{
         search = self.search
         search['name'] = n
 
-        f = unicode(self.find.toPlainText())
+        f = unicode_type(self.find.toPlainText())
         if not f:
             error_dialog(self, _('Must specify find'), _(
                 'You must specify a find expression'), show=True)
@@ -768,7 +839,7 @@ class EditSearch(QFrame):  # {{{
                     'You must specify a function name in Function-Regex mode'), show=True)
                 return False
         else:
-            r = unicode(self.replace.toPlainText())
+            r = unicode_type(self.replace.toPlainText())
         search['replace'] = r
 
         search['dot_all'] = bool(self.dot_all.isChecked())
@@ -835,15 +906,19 @@ class SavedSearches(QWidget):
         self.delegate = SearchDelegate(searches)
         searches.setItemDelegate(self.delegate)
         searches.setAlternatingRowColors(True)
+        searches.setDragEnabled(True), searches.setAcceptDrops(True), searches.setDragDropMode(searches.InternalMove)
+        searches.setDropIndicatorShown(True)
         h.addLayout(stack, stretch=10)
         self.v = v = QVBoxLayout()
         h.addLayout(v)
         l.addLayout(h)
         stack.currentChanged.connect(self.stack_current_changed)
 
-        def pb(text, tooltip=None):
+        def pb(text, tooltip=None, action=None):
             b = AnimatablePushButton(text, self)
             b.setToolTip(tooltip or '')
+            if action:
+                b.setObjectName(action)
             b.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
             return b
 
@@ -857,9 +932,9 @@ class SavedSearches(QWidget):
                 (_('Replace &all'), 'replace-all', _('Run Replace All for all selected entries in the order selected')),
                 (_('&Count all'), 'count', _('Run Count All for all selected entries')),
         ]:
-            self.action_button_map[action] = b = pb(text, tooltip)
+            self.action_button_map[action] = b = pb(text, tooltip, action)
             v.addWidget(b)
-            b.clicked.connect(partial(self.run_search, action))
+            connect_lambda(b.clicked, self, lambda self: self.run_search(self.sender().objectName()))
 
         self.d1 = d = QFrame(self)
         d.setFrameStyle(QFrame.HLine)
@@ -867,11 +942,20 @@ class SavedSearches(QWidget):
 
         self.h3 = h = QHBoxLayout()
         self.upb = b = QToolButton(self)
-        b.setIcon(QIcon(I('arrow-up.png'))), b.setToolTip(_('Move selected entries up'))
-        b.clicked.connect(partial(self.move_entry, -1))
+        self.move_up_action = a = QAction(self)
+        a.setShortcut(QKeySequence('Alt+Up'))
+        b.setIcon(QIcon(I('arrow-up.png'))), b.setToolTip(_('Move selected entries up') + ' [%s]' % a.shortcut().toString(QKeySequence.NativeText))
+        connect_lambda(a.triggered, self, lambda self: self.move_entry(-1))
+        self.searches.addAction(a)
+        connect_lambda(b.clicked, self, lambda self: self.move_entry(-1))
+
         self.dnb = b = QToolButton(self)
-        b.setIcon(QIcon(I('arrow-down.png'))), b.setToolTip(_('Move selected entries down'))
-        b.clicked.connect(partial(self.move_entry, 1))
+        self.move_down_action = a = QAction(self)
+        a.setShortcut(QKeySequence('Alt+Down'))
+        b.setIcon(QIcon(I('arrow-down.png'))), b.setToolTip(_('Move selected entries down') + ' [%s]' % a.shortcut().toString(QKeySequence.NativeText))
+        connect_lambda(a.triggered, self, lambda self: self.move_entry(1))
+        self.searches.addAction(a)
+        connect_lambda(b.clicked, self, lambda self: self.move_entry(1))
         h.addWidget(self.upb), h.addWidget(self.dnb)
         v.addLayout(h)
 
@@ -918,8 +1002,8 @@ class SavedSearches(QWidget):
         self.eb2 = b = pb(_('E&xport'), _('Export saved searches'))
         v.addWidget(b)
         self.em = m = QMenu(_('Export'))
-        m.addAction(_('Export All'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=True)))
-        m.addAction(_('Export Selected'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=False)))
+        m.addAction(_('Export all'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=True)))
+        m.addAction(_('Export selected'), lambda : QTimer.singleShot(0, partial(self.export_searches, all=False)))
         b.setMenu(m)
 
         self.searches.setFocus(Qt.OtherFocusReason)
@@ -1033,16 +1117,22 @@ class SavedSearches(QWidget):
     def move_entry(self, delta):
         if self.editing_search:
             return
-        rows = {index.row() for index in self.searches.selectionModel().selectedIndexes()} - {-1}
+        sm = self.searches.selectionModel()
+        rows = {index.row() for index in sm.selectedIndexes()} - {-1}
         if rows:
+            searches = [self.model.search_for_index(index) for index in sm.selectedIndexes()]
+            current_search = self.model.search_for_index(self.searches.currentIndex())
             with tprefs:
                 for row in sorted(rows, reverse=delta > 0):
                     self.model.move_entry(row, delta)
-            nrow = row + delta
-            index = self.model.index(nrow)
-            if index.isValid():
-                sm = self.searches.selectionModel()
-                sm.setCurrentIndex(index, sm.ClearAndSelect)
+            sm.clear()
+            for s in searches:
+                index = self.model.index_for_search(s)
+                if index.isValid() and index.row() > -1:
+                    if s is current_search:
+                        sm.setCurrentIndex(index, sm.Select)
+                    else:
+                        sm.select(index, sm.Select)
 
     def search_editing_done(self, save_changes):
         if save_changes and not self.edit_search_widget.save_changes():
@@ -1117,7 +1207,7 @@ class SavedSearches(QWidget):
 
     def import_searches(self):
         path = choose_files(self, 'import_saved_searches', _('Choose file'), filters=[
-            (_('Saved Searches'), ['json'])], all_files=False, select_only_single_file=True)
+            (_('Saved searches'), ['json'])], all_files=False, select_only_single_file=True)
         if path:
             with open(path[0], 'rb') as f:
                 obj = json.loads(f.read())
@@ -1130,7 +1220,7 @@ class SavedSearches(QWidget):
                 return err()
             searches = []
             for item in obj['searches']:
-                if not isinstance(item, dict) or not set(item.iterkeys()).issuperset(needed_keys):
+                if not isinstance(item, dict) or not set(item).issuperset(needed_keys):
                     return err
                 searches.append({k:item[k] for k in needed_keys})
 
@@ -1159,7 +1249,7 @@ class SavedSearches(QWidget):
                     'No searches selected'), show=True)
         [s.__setitem__('mode', s.get('mode', 'regex')) for s in searches]
         path = choose_save_file(self, 'export-saved-searches', _('Choose file'), filters=[
-            (_('Saved Searches'), ['json'])], all_files=False)
+            (_('Saved searches'), ['json'])], all_files=False)
         if path:
             if not path.lower().endswith('.json'):
                 path += '.json'
@@ -1174,7 +1264,7 @@ def validate_search_request(name, searchable_names, has_marked_text, state, gui_
     if name is None and where in {'current', 'selected-text'}:
         err = _('No file is being edited.')
     elif where == 'selected' and not searchable_names['selected']:
-        err = _('No files are selected in the Files Browser')
+        err = _('No files are selected in the File browser')
     elif where == 'selected-text' and not has_marked_text:
         err = _('No text is marked. First select some text, and then use'
                 ' The "Mark selected text" action in the Search menu to mark it.')
@@ -1189,7 +1279,7 @@ def validate_search_request(name, searchable_names, has_marked_text, state, gui_
 class InvalidRegex(regex.error):
 
     def __init__(self, raw, e):
-        regex.error.__init__(self, e.message)
+        regex.error.__init__(self, error_message(e))
         self.regex = raw
 
 
@@ -1205,16 +1295,34 @@ def get_search_regex(state):
         flags |= regex.DOTALL
     if state['direction'] == 'up':
         flags |= regex.REVERSE
-    ans = regex_cache.get((flags, raw), None)
-    if ans is None:
-        try:
-            ans = regex_cache[(flags, raw)] = regex.compile(raw, flags=flags)
-        except regex.error as e:
-            raise InvalidRegex(raw, e)
+    try:
+        ans = compile_regular_expression(raw, flags=flags)
+    except regex.error as e:
+        raise InvalidRegex(raw, e)
 
     return ans
 
 
+def get_search_function(state):
+    ans = state['replace']
+    is_regex = state['mode'] != 'normal'
+    if not is_regex:
+        # We dont want backslash escape sequences interpreted in normal mode
+        return lambda m: ans
+    if state['mode'] == 'function':
+        try:
+            return replace_functions()[ans]
+        except KeyError:
+            if not ans:
+                return Function('empty-function', '')
+            raise NoSuchFunction(ans)
+    return ans
+
+
+def get_search_name(state):
+    return state.get('name', state['find'])
+
+
 def initialize_search_request(state, action, current_editor, current_editor_name, searchable_names):
     editor = None
     where = state['where']
@@ -1255,18 +1363,6 @@ class NoSuchFunction(ValueError):
     pass
 
 
-def get_search_function(search):
-    ans = search['replace']
-    if search['mode'] == 'function':
-        try:
-            return replace_functions()[ans]
-        except KeyError:
-            if not ans:
-                return Function('empty-function', '')
-            raise NoSuchFunction(ans)
-    return ans
-
-
 def show_function_debug_output(func):
     if isinstance(func, Function):
         val = func.debug_buf.getvalue().strip()
@@ -1296,15 +1392,17 @@ def run_search(
     if len(searches) > 1:
         errfind = _('the selected searches')
 
+    search_names = [get_search_name(search) for search in searches]
+
     try:
         searches = [(get_search_regex(search), get_search_function(search)) for search in searches]
     except InvalidRegex as e:
         return error_dialog(gui_parent, _('Invalid regex'), '<p>' + _(
             'The regular expression you entered is invalid: <pre>{0}</pre>With error: {1}').format(
-                prepare_string_for_xml(e.regex), e.message), show=True)
+                prepare_string_for_xml(e.regex), error_message(e)), show=True)
     except NoSuchFunction as e:
         return error_dialog(gui_parent, _('No such function'), '<p>' + _(
-            'No replace function with the name: %s exists') % prepare_string_for_xml(e.message), show=True)
+            'No replace function with the name: %s exists') % prepare_string_for_xml(error_message(e)), show=True)
 
     def no_match():
         QApplication.restoreOverrideCursor()
@@ -1323,7 +1421,7 @@ def run_search(
                     return True
                 if wrap and not files and editor.find(p, wrap=True, marked=marked, save_match='gui'):
                     return True
-            for fname, syntax in files.iteritems():
+            for fname, syntax in iteritems(files):
                 ed = editors.get(fname, None)
                 if ed is not None:
                     if not wrap and ed is editor:
@@ -1345,7 +1443,7 @@ def run_search(
             prefix += ' '
         error_dialog(
             gui_parent, _('Cannot replace'), prefix + _(
-            'You must first click Find, before trying to replace'), show=True)
+            'You must first click "Find", before trying to replace'), show=True)
         return False
 
     def do_replace():
@@ -1363,24 +1461,32 @@ def run_search(
         return no_replace(_(
                 'Currently selected text does not match the search query.'))
 
-    def count_message(replaced, count, show_diff=False, show_dialog=True):
+    def count_message(replaced, count, show_diff=False, show_dialog=True, count_map=None):
         if show_dialog:
             if replaced:
                 msg = _('Performed the replacement at {num} occurrences of {query}')
             else:
                 msg = _('Found {num} occurrences of {query}')
             msg = msg.format(num=count, query=errfind)
+            det_msg = ''
+            if count_map is not None and count > 0 and len(count_map) > 1:
+                for k in sorted(count_map):
+                    det_msg += _('{0}: {1} occurrences').format(k, count_map[k]) + '\n'
             if show_diff and count > 0:
-                d = MessageBox(MessageBox.INFO, _('Searching done'), prepare_string_for_xml(msg), parent=gui_parent, show_copy_button=False)
-                d.diffb = b = d.bb.addButton(_('See what &changed'), d.bb.ActionRole)
-                b.setIcon(QIcon(I('diff.png'))), d.set_details(None), b.clicked.connect(d.accept)
-                b.clicked.connect(partial(show_current_diff, allow_revert=True), type=Qt.QueuedConnection)
+                d = MessageBox(MessageBox.INFO, _('Searching done'), prepare_string_for_xml(msg), parent=gui_parent, show_copy_button=False, det_msg=det_msg)
+                d.diffb = b = d.bb.addButton(_('See what &changed'), d.bb.AcceptRole)
+                d.show_changes = False
+                b.setIcon(QIcon(I('diff.png'))), b.clicked.connect(d.accept)
+                connect_lambda(b.clicked, d, lambda d: setattr(d, 'show_changes', True))
                 d.exec_()
+                if d.show_changes:
+                    show_current_diff(allow_revert=True)
             else:
-                info_dialog(gui_parent, _('Searching done'), prepare_string_for_xml(msg), show=True)
+                info_dialog(gui_parent, _('Searching done'), prepare_string_for_xml(msg), show=True, det_msg=det_msg)
 
     def do_all(replace=True):
         count = 0
+        count_map = Counter()
         if not files and editor is None:
             return 0
         lfiles = files or {current_editor_name:editor.syntax}
@@ -1393,7 +1499,7 @@ def run_search(
                 raw = current_container().raw_data(n)
             raw_data[n] = raw
 
-        for p, repl in searches:
+        for search_name, (p, repl) in zip(search_names, searches):
             repl_is_func = isinstance(repl, Function)
             file_iterator = lfiles
             if repl_is_func:
@@ -1412,6 +1518,7 @@ def run_search(
                 else:
                     num = len(p.findall(raw))
                 count += num
+                count_map[search_name] += num
             if repl_is_func:
                 repl.end()
                 show_function_debug_output(repl)
@@ -1424,7 +1531,7 @@ def run_search(
                 with current_container().open(n, 'wb') as f:
                     f.write(raw.encode('utf-8'))
         QApplication.restoreOverrideCursor()
-        count_message(replace, count, show_diff=replace)
+        count_message(replace, count, show_diff=replace, count_map=count_map)
         return count
 
     with BusyCursor():
