@@ -8,18 +8,20 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 from calibre import prepare_string_for_xml
 from calibre.gui2.tweak_book import current_container, tprefs
+from polyglot.builtins import iteritems
 
 DEFAULT_TEMPLATES = {
     'html':
 '''\
 <?xml version='1.0' encoding='utf-8'?>
 <html xmlns="http://www.w3.org/1999/xhtml">
-    <head>
-        <title>{TITLE}</title>
-    </head>
-    <body>
-        %CURSOR%
-    </body>
+<head>
+    <title>{TITLE}</title>
+</head>
+
+<body>
+    %CURSOR%
+</body>
 </html>
 ''',
 
@@ -45,5 +47,4 @@ def template_for(syntax):
         'AUTHOR': ' & '.join(mi.authors),
     }
     return raw_template_for(syntax).format(
-        **{k:prepare_string_for_xml(v, True) for k, v in data.iteritems()})
-
+        **{k:prepare_string_for_xml(v, True) for k, v in iteritems(data)})
