@@ -7,7 +7,6 @@ __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os, textwrap, unicodedata
-from itertools import izip
 from collections import OrderedDict
 
 from PyQt5.Qt import (
@@ -24,10 +23,11 @@ from calibre.ebooks.oeb.polish.cover import get_raster_cover_name
 from calibre.ebooks.oeb.polish.utils import lead_text, guess_type
 from calibre.gui2 import error_dialog, choose_files, choose_save_file, info_dialog, choose_images
 from calibre.gui2.tweak_book import tprefs, current_container
-from calibre.gui2.widgets2 import Dialog as BaseDialog
+from calibre.gui2.widgets2 import Dialog as BaseDialog, HistoryComboBox
 from calibre.utils.icu import primary_sort_key, sort_key, primary_contains, numeric_sort_key
 from calibre.utils.matcher import get_char, Matcher
 from calibre.gui2.complete2 import EditWithComplete
+from polyglot.builtins import iteritems, unicode_type, zip
 
 ROOT = QModelIndex()
 PARAGRAPH_SEPARATOR = '\u2029'
@@ -71,13 +71,13 @@ class InsertTag(Dialog):  # {{{
 
     @property
     def tag(self):
-        return unicode(self.tag_input.text()).strip()
+        return unicode_type(self.tag_input.text()).strip()
 
     @classmethod
     def test(cls):
         d = cls()
         if d.exec_() == d.Accepted:
-            print (d.tag)
+            print(d.tag)
 
 # }}}
 
@@ -132,7 +132,7 @@ class RationalizeFolders(Dialog):  # {{{
     def folder_map(self):
         ans = {}
         for typ, x in self.TYPE_MAP:
-            val = unicode(getattr(self, '%s_folder' % typ).text()).strip().strip('/')
+            val = unicode_type(getattr(self, '%s_folder' % typ).text()).strip().strip('/')
             ans[typ] = val
         return ans
 
@@ -244,15 +244,15 @@ class ImportForeign(Dialog):  # {{{
             self.dest.setText(path)
 
     def accept(self):
-        if not unicode(self.src.text()):
+        if not unicode_type(self.src.text()):
             return error_dialog(self, _('Need document'), _(
                 'You must specify the source file that will be imported.'), show=True)
         Dialog.accept(self)
 
     @property
     def data(self):
-        src = unicode(self.src.text()).strip()
-        dest = unicode(self.dest.text()).strip()
+        src = unicode_type(self.src.text()).strip()
+        dest = unicode_type(self.dest.text()).strip()
         if not dest:
             dest = src.rpartition('.')[0] + '.epub'
         return src, dest
@@ -347,7 +347,7 @@ class Results(QWidget):
             [(p.setTextFormat(Qt.RichText), p.setTextOption(self.text_option)) for p in prefixes]
             self.maxwidth = max([x.size().width() for x in prefixes])
             self.results = tuple((prefix, self.make_text(text, positions), text)
-                for prefix, (text, positions) in izip(prefixes, results.iteritems()))
+                for prefix, (text, positions) in zip(prefixes, iteritems(results)))
         else:
             self.results = ()
             self.current_result = -1
@@ -450,7 +450,7 @@ class QuickOpen(Dialog):
         l.addWidget(self.bb, alignment=Qt.AlignBottom)
 
     def update_matches(self, text):
-        text = unicode(text).strip()
+        text = unicode_type(text).strip()
         self.help_label.setVisible(False)
         self.results.setVisible(True)
         matches = self.matcher(text, limit=100)
@@ -475,7 +475,7 @@ class QuickOpen(Dialog):
         items = get_items_from_dir(os.getcwdu(), lambda x:not x.endswith('.pyc'))
         d = cls(items)
         d.exec_()
-        print (d.selected_result)
+        print(d.selected_result)
 
 # }}}
 
@@ -546,12 +546,12 @@ class NamesModel(QAbstractListModel):
             return '\xa0' * 20
 
     def filter(self, query):
-        query = unicode(query or '')
+        query = unicode_type(query or '')
         self.beginResetModel()
         if not query:
             self.items = tuple((text, None) for text in self.names)
         else:
-            self.items = tuple(self.matcher(query).iteritems())
+            self.items = tuple(iteritems(self.matcher(query)))
         self.endResetModel()
         self.filtered.emit(not bool(query))
 
@@ -611,7 +611,7 @@ class AnchorsModel(QAbstractListModel):
         self.filter('')
 
     def filter(self, query):
-        query = unicode(query or '')
+        query = unicode_type(query or '')
         self.beginResetModel()
         self.items = [x for x in self.names if primary_contains(query, x[0]) or primary_contains(query, x[1])]
         self.endResetModel()
@@ -671,10 +671,19 @@ class InsertLink(Dialog):
         t.setText(self.initial_text or '')
         t.setPlaceholderText(_('The (optional) text for the link'))
 
-        self.template_edit = t = QLineEdit(self)
+        self.template_edit = t = HistoryComboBox(self)
+        t.lineEdit().setClearButtonEnabled(True)
+        t.initialize('edit_book_insert_link_template_history')
         tl.addRow(_('Tem&plate:'), t)
         from calibre.gui2.tweak_book.editor.smarts.html import DEFAULT_LINK_TEMPLATE
         t.setText(tprefs.get('insert-hyperlink-template', None) or DEFAULT_LINK_TEMPLATE)
+        t.setToolTip('<p>' + _('''
+            The template to use for generating the link. In addition to {0} and {1}
+            you can also use {2}, {3} and {4} variables
+            in the template, they will be replaced by the source filename, the destination
+            filename and the anchor, respectively.
+        ''').format(
+            '_TITLE_', '_TARGET', '_SOURCE_FILENAME_', '_DEST_FILENAME_', '_ANCHOR_'))
 
         l.addWidget(self.bb)
 
@@ -730,16 +739,32 @@ class InsertLink(Dialog):
 
     @property
     def href(self):
-        return unicode(self.target.text()).strip()
+        return unicode_type(self.target.text()).strip()
 
     @property
     def text(self):
-        return unicode(self.text_edit.text()).strip()
+        return unicode_type(self.text_edit.text()).strip()
 
     @property
     def template(self):
         return self.template_edit.text().strip() or None
 
+    @property
+    def rendered_template(self):
+        ans = self.template
+        if ans:
+            target = self.href
+            frag = target.partition('#')[-1]
+            if target.startswith('#'):
+                target = ''
+            else:
+                target = target.split('#', 1)[0]
+                target = self.container.href_to_name(target)
+            ans = ans.replace('_SOURCE_FILENAME_', self.source_name or '')
+            ans = ans.replace('_DEST_FILENAME_', target or '')
+            ans = ans.replace('_ANCHOR_', frag or '')
+        return ans
+
     @classmethod
     def test(cls):
         import sys
@@ -795,7 +820,7 @@ class InsertSemantics(Dialog):
             'text': _('First "real" page of content'),
         }
         t = _
-        all_types = [(k, (('%s (%s)' % (t(v), type_map_help[k])) if k in type_map_help else t(v))) for k, v in self.known_type_map.iteritems()]
+        all_types = [(k, (('%s (%s)' % (t(v), type_map_help[k])) if k in type_map_help else t(v))) for k, v in iteritems(self.known_type_map)]
         all_types.sort(key=lambda x: sort_key(x[1]))
         self.all_types = OrderedDict(all_types)
 
@@ -805,7 +830,7 @@ class InsertSemantics(Dialog):
 
         self.tl = tl = QFormLayout()
         self.semantic_type = QComboBox(self)
-        for key, val in self.all_types.iteritems():
+        for key, val in iteritems(self.all_types):
             self.semantic_type.addItem(val, key)
         tl.addRow(_('Type of &semantics:'), self.semantic_type)
         self.target = t = QLineEdit(self)
@@ -858,7 +883,7 @@ class InsertSemantics(Dialog):
         d.exec_()
 
     def semantic_type_changed(self):
-        item_type = unicode(self.semantic_type.itemData(self.semantic_type.currentIndex()) or '')
+        item_type = unicode_type(self.semantic_type.itemData(self.semantic_type.currentIndex()) or '')
         name, frag = self.final_type_map.get(item_type, (None, None))
         self.show_type(name, frag)
 
@@ -883,8 +908,8 @@ class InsertSemantics(Dialog):
         self.target.blockSignals(False)
 
     def target_text_changed(self):
-        name, frag = unicode(self.target.text()).partition('#')[::2]
-        item_type = unicode(self.semantic_type.itemData(self.semantic_type.currentIndex()) or '')
+        name, frag = unicode_type(self.target.text()).partition('#')[::2]
+        item_type = unicode_type(self.semantic_type.itemData(self.semantic_type.currentIndex()) or '')
         self.final_type_map[item_type] = (name, frag or None)
 
     def selected_file_changed(self, *args):
@@ -921,13 +946,13 @@ class InsertSemantics(Dialog):
 
     @property
     def changed_type_map(self):
-        return {k:v for k, v in self.final_type_map.iteritems() if v != self.original_type_map.get(k, None)}
+        return {k:v for k, v in iteritems(self.final_type_map) if v != self.original_type_map.get(k, None)}
 
     def apply_changes(self, container):
         from calibre.ebooks.oeb.polish.opf import set_guide_item, get_book_language
         from calibre.translations.dynamic import translate
         lang = get_book_language(container)
-        for item_type, (name, frag) in self.changed_type_map.iteritems():
+        for item_type, (name, frag) in iteritems(self.changed_type_map):
             title = self.known_type_map[item_type]
             if lang:
                 title = translate(lang, title)
@@ -1002,7 +1027,7 @@ class FilterCSS(Dialog):  # {{{
             a('float'), a('clear')
         if self.opt_colors.isChecked():
             a('color'), a('background-color')
-        for x in unicode(self.others.text()).split(','):
+        for x in unicode_type(self.others.text()).split(','):
             x = x.strip()
             if x:
                 a(x)
@@ -1012,7 +1037,7 @@ class FilterCSS(Dialog):  # {{{
     def test(cls):
         d = cls()
         if d.exec_() == d.Accepted:
-            print (d.filtered_properties)
+            print(d.filtered_properties)
 
 # }}}
 
@@ -1068,7 +1093,7 @@ class AddCover(Dialog):
     @property
     def image_names(self):
         img_types = {guess_type('a.'+x) for x in ('png', 'jpeg', 'gif')}
-        for name, mt in self.container.mime_map.iteritems():
+        for name, mt in iteritems(self.container.mime_map):
             if mt.lower() in img_types:
                 yield name
 
@@ -1188,7 +1213,7 @@ class PlainTextEdit(QPlainTextEdit):  # {{{
         return ans.rstrip('\0')
 
     def selected_text_from_cursor(self, cursor):
-        return unicodedata.normalize('NFC', unicode(cursor.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0'))
+        return unicodedata.normalize('NFC', unicode_type(cursor.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0'))
 
     @property
     def selected_text(self):
