@@ -8,6 +8,7 @@ __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os
 from calibre.srv.tests.base import BaseTest
+from polyglot.builtins import itervalues, filter
 
 
 class TestRouter(BaseTest):
@@ -52,7 +53,7 @@ class TestRouter(BaseTest):
         router = Router()
 
         def find(path):
-            path = filter(None, path.split('/'))
+            path = list(filter(None, path.split('/')))
             ep, args = router.find_route(path)
             args = list(args)
             return ep, args
@@ -85,7 +86,7 @@ class TestRouter(BaseTest):
         def get(ctx, dest, a, b):
             pass
 
-        for x in locals().itervalues():
+        for x in itervalues(locals()):
             if getattr(x, 'is_endpoint', False):
                 router.add(x)
         router.finalize()
