@@ -2,7 +2,6 @@
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 from __future__ import (unicode_literals, division, absolute_import,
                         print_function)
-from future_builtins import map
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -12,10 +11,14 @@ __docformat__ = 'restructuredtext en'
 Test a binary calibre build to ensure that all needed binary images/libraries have loaded.
 '''
 
-import os, ctypes, sys, unittest
-from calibre.constants import plugins, iswindows, islinux, isosx
+import os, ctypes, sys, unittest, time
+
+from calibre.constants import plugins, iswindows, islinux, isosx, ispy3
+from polyglot.builtins import iteritems, map, unicode_type
+
 is_ci = os.environ.get('CI', '').lower() == 'true'
 
+
 class BuildTest(unittest.TestCase):
 
     @unittest.skipUnless(iswindows and not is_ci, 'DLL loading needs testing only on windows (non-continuous integration)')
@@ -42,6 +45,21 @@ class BuildTest(unittest.TestCase):
     def test_regex(self):
         import regex
         self.assertEqual(regex.findall(r'(?i)(a)(b)', 'ab cd AB 1a1b'), [('a', 'b'), ('A', 'B')])
+        self.assertEqual(regex.escape('a b', literal_spaces=True), 'a b')
+
+    def test_chardet(self):
+        from chardet import detect
+        raw = 'mūsi Füße'.encode('utf-8')
+        data = detect(raw)
+        self.assertEqual(data['encoding'], 'utf-8')
+        self.assertGreater(data['confidence'], 0.5)
+        # The following is used by html5lib
+        from chardet.universaldetector import UniversalDetector
+        detector = UniversalDetector()
+        self.assertTrue(hasattr(detector, 'done'))
+        detector.feed(raw)
+        detector.close()
+        self.assertEqual(detector.result['encoding'], 'utf-8')
 
     def test_lzma(self):
         from lzma.xz import test_lzma2
@@ -50,13 +68,23 @@ class BuildTest(unittest.TestCase):
     def test_html5lib(self):
         import html5lib.html5parser  # noqa
         from html5lib import parse  # noqa
-        # Test that we are using the calibre version of html5lib
-        from calibre.ebooks.oeb.polish.parsing import parse_html5
-        parse_html5('<p>xxx')
 
-    def test_spell(self):
-        from calibre.spell.dictionary import test_dictionaries
-        test_dictionaries()
+    def test_html5_parser(self):
+        from html5_parser import parse
+        parse('<p>xxx')
+
+    def test_bs4(self):
+        import soupsieve, bs4
+        del soupsieve, bs4
+
+    def test_zeroconf(self):
+        if ispy3:
+            import zeroconf as z, ifaddr
+        else:
+            import calibre.utils.Zeroconf as z
+            ifaddr = None
+        del z
+        del ifaddr
 
     def test_plugins(self):
         exclusions = set()
@@ -84,7 +112,7 @@ class BuildTest(unittest.TestCase):
         from calibre.utils.cleantext import test_clean_xml_chars
         test_clean_xml_chars()
         from lxml import etree
-        raw = '<a/>'
+        raw = b'<a/>'
         root = etree.fromstring(raw)
         self.assertEqual(etree.tostring(root), raw)
 
@@ -92,6 +120,17 @@ class BuildTest(unittest.TestCase):
         from calibre.utils.certgen import create_key_pair
         create_key_pair()
 
+    def test_msgpack(self):
+        from calibre.utils.serialize import msgpack_dumps, msgpack_loads
+        from calibre.utils.date import utcnow
+        for obj in ({1:1}, utcnow()):
+            s = msgpack_dumps(obj)
+            self.assertEqual(obj, msgpack_loads(s))
+        self.assertEqual(type(msgpack_loads(msgpack_dumps(b'b'))), bytes)
+        self.assertEqual(type(msgpack_loads(msgpack_dumps(u'b'))), unicode_type)
+        large = b'x' * (100 * 1024 * 1024)
+        msgpack_loads(msgpack_dumps(large))
+
     @unittest.skipUnless(isosx, 'FSEvents only present on OS X')
     def test_fsevents(self):
         from fsevents import Observer, Stream
@@ -100,9 +139,32 @@ class BuildTest(unittest.TestCase):
     @unittest.skipUnless(iswindows, 'winutil is windows only')
     def test_winutil(self):
         from calibre.constants import plugins
+        from calibre import strftime
         winutil = plugins['winutil'][0]
+
+        def au(x, name):
+            self.assertTrue(isinstance(x, unicode_type), name + '() did not return a unicode string')
         for x in winutil.argv():
-            self.assertTrue(isinstance(x, unicode), 'argv() not returning unicode string')
+            au(x, 'argv')
+        for x in 'username temp_path locale_name'.split():
+            au(getattr(winutil, x)(), x)
+        d = winutil.localeconv()
+        au(d['thousands_sep'], 'localeconv')
+        au(d['decimal_point'], 'localeconv')
+        for k, v in iteritems(d):
+            au(v, k)
+        for k in os.environ.keys():
+            au(winutil.getenv(unicode_type(k)), 'getenv-' + k)
+        os.environ['XXXTEST'] = 'YYY'
+        self.assertEqual(winutil.getenv(u'XXXTEST'), u'YYY')
+        del os.environ['XXXTEST']
+        self.assertIsNone(winutil.getenv(u'XXXTEST'))
+        t = time.localtime()
+        fmt = u'%Y%a%b%e%H%M'
+        for fmt in (fmt, fmt.encode('ascii')):
+            x = strftime(fmt, t)
+            au(x, 'strftime')
+            self.assertEqual(unicode_type(time.strftime(fmt.replace('%e', '%#d'), t)), x)
 
     def test_sqlite(self):
         import sqlite3
@@ -115,6 +177,7 @@ class BuildTest(unittest.TestCase):
         conn = apsw.Connection(':memory:')
         conn.close()
 
+    @unittest.skipIf('SKIP_QT_BUILD_TEST' in os.environ, 'Skipping Qt build test as it causes crashes in the macOS VM')
     def test_qt(self):
         from PyQt5.Qt import QImageReader, QNetworkAccessManager, QFontDatabase
         from calibre.utils.img import image_from_data, image_to_data, test
@@ -125,10 +188,10 @@ class BuildTest(unittest.TestCase):
         # it should just work because the hard-coded paths of the Qt
         # installation should work. If they do not, then it is a distro
         # problem.
-        fmts = set(map(unicode, QImageReader.supportedImageFormats()))
+        fmts = set(map(lambda x: x.data().decode('utf-8'), QImageReader.supportedImageFormats()))
         testf = {'jpg', 'png', 'svg', 'ico', 'gif'}
         self.assertEqual(testf.intersection(fmts), testf, "Qt doesn't seem to be able to load some of its image plugins. Available plugins: %s" % fmts)
-        data = I('blank.png', allow_user_override=False, data=True)
+        data = P('images/blank.png', allow_user_override=False, data=True)
         img = image_from_data(data)
         image_from_data(P('catalog/mastheadImage.gif', allow_user_override=False, data=True))
         for fmt in 'png bmp jpeg'.split():
@@ -139,8 +202,12 @@ class BuildTest(unittest.TestCase):
 
         from calibre.gui2 import Application
         os.environ.pop('DISPLAY', None)
-        app = Application([], headless=islinux)
+        has_headless = isosx or islinux
+        app = Application([], headless=has_headless)
         self.assertGreaterEqual(len(QFontDatabase().families()), 5, 'The QPA headless plugin is not able to locate enough system fonts via fontconfig')
+        if has_headless:
+            from calibre.ebooks.covers import create_cover
+            create_cover('xxx', ['yyy'])
         na = QNetworkAccessManager()
         self.assertTrue(hasattr(na, 'sslErrors'), 'Qt not compiled with openssl')
         from PyQt5.QtWebKitWidgets import QWebView
@@ -163,7 +230,7 @@ class BuildTest(unittest.TestCase):
         i = Image.open(I('lt.png', allow_user_override=False))
         self.assertGreaterEqual(i.size, (20, 20))
 
-    @unittest.skipUnless(iswindows, 'File dialog helper only used on windows')
+    @unittest.skipUnless(iswindows and not is_ci, 'File dialog helper only used on windows (non-continuous-itegration)')
     def test_file_dialog_helper(self):
         from calibre.gui2.win_file_dialogs import test
         test()
@@ -200,7 +267,7 @@ class BuildTest(unittest.TestCase):
 
     def test_netifaces(self):
         import netifaces
-        self.assertGreaterEqual(netifaces.interfaces(), 1, 'netifaces could find no network interfaces')
+        self.assertGreaterEqual(len(netifaces.interfaces()), 1, 'netifaces could find no network interfaces')
 
     def test_psutil(self):
         import psutil
@@ -216,11 +283,24 @@ class BuildTest(unittest.TestCase):
         import readline
         del readline
 
+    def test_html2text(self):
+        import html2text
+        del html2text
+
     def test_markdown(self):
-        from calibre.ebooks.markdown import Markdown
-        Markdown(extensions=['extra'])
-        from calibre.library.comments import sanitize_html
-        sanitize_html(b'''<script>moo</script>xxx<img src="http://moo.com/x.jpg">''')
+        from calibre.ebooks.txt.processor import create_markdown_object
+        from calibre.ebooks.conversion.plugins.txt_input import MD_EXTENSIONS
+        create_markdown_object(sorted(MD_EXTENSIONS))
+        from calibre.library.comments import sanitize_comments_html
+        sanitize_comments_html(b'''<script>moo</script>xxx<img src="http://moo.com/x.jpg">''')
+
+    def test_feedparser(self):
+        from calibre.web.feeds.feedparser import parse
+        # sgmllib is needed for feedparser parsing malformed feeds
+        # on python3 you can get it by taking it from python2 stdlib and
+        # running 2to3 on it
+        import sgmllib
+        sgmllib, parse
 
     def test_openssl(self):
         import ssl
@@ -230,6 +310,7 @@ class BuildTest(unittest.TestCase):
             if not cafile or not cafile.endswith('/mozilla-ca-certs.pem') or not os.access(cafile, os.R_OK):
                 self.assert_('Mozilla CA certs not loaded')
 
+
 def find_tests():
     ans = unittest.defaultTestLoader.loadTestsFromTestCase(BuildTest)
     from calibre.utils.icu_test import find_tests
@@ -238,11 +319,15 @@ def find_tests():
     ans.addTests(unittest.defaultTestLoader.loadTestsFromModule(dtests))
     from tinycss.tests.main import find_tests
     ans.addTests(find_tests())
+    from calibre.spell.dictionary import find_tests
+    ans.addTests(find_tests())
     return ans
 
+
 def test():
     from calibre.utils.run_tests import run_cli
     run_cli(find_tests())
 
+
 if __name__ == '__main__':
     test()
