@@ -6,16 +6,18 @@ from __future__ import (unicode_literals, division, absolute_import,
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, cPickle, sys
+import os, sys
 from threading import Thread
 from collections import namedtuple
-from Queue import Queue
 
 from calibre import detect_ncpus, as_unicode, prints
 from calibre.constants import iswindows, DEBUG
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils import join_with_timeout
 from calibre.utils.ipc import eintr_retry_call
+from calibre.utils.serialize import msgpack_dumps, msgpack_loads, pickle_dumps, pickle_loads
+from polyglot.builtins import iteritems, itervalues
+from polyglot.queue import Queue
 
 Job = namedtuple('Job', 'id module func args kwargs')
 Result = namedtuple('Result', 'value err traceback')
@@ -49,6 +51,7 @@ if iswindows:
             from calibre.utils.ipc.launch import windows_null_file
             worker_kwargs['stdout'] = worker_kwargs['stderr'] = windows_null_file
 
+
 def get_stdout(process):
     import time
     while process.poll() is None:
@@ -64,6 +67,7 @@ def get_stdout(process):
         except (EOFError, EnvironmentError):
             break
 
+
 def start_worker(code, name=''):
     from calibre.utils.ipc.simple_worker import start_pipe_worker
     if name:
@@ -75,6 +79,7 @@ def start_worker(code, name=''):
         t.start()
     return p
 
+
 class Failure(Exception):
 
     def __init__(self, tf):
@@ -83,6 +88,7 @@ class Failure(Exception):
         self.job_id = tf.job_id
         self.failure_message = tf.message
 
+
 class Worker(object):
 
     def __init__(self, p, conn, events, name):
@@ -91,7 +97,7 @@ class Worker(object):
         self.name = name or ''
 
     def __call__(self, job):
-        eintr_retry_call(self.conn.send_bytes, cPickle.dumps(job, -1))
+        eintr_retry_call(self.conn.send_bytes, pickle_dumps(job))
         if job is not None:
             self.job_id = job.id
             t = Thread(target=self.recv, name='PoolWorker-'+self.name)
@@ -100,7 +106,7 @@ class Worker(object):
 
     def recv(self):
         try:
-            result = cPickle.loads(eintr_retry_call(self.conn.recv_bytes))
+            result = pickle_loads(eintr_retry_call(self.conn.recv_bytes))
             wr = WorkerResult(self.job_id, result, False, self)
         except Exception as err:
             import traceback
@@ -126,7 +132,7 @@ class Pool(Thread):
         self.results = Queue()
         self.tracker = Queue()
         self.terminal_failure = None
-        self.common_data = cPickle.dumps(None, -1)
+        self.common_data = pickle_dumps(None)
         self.worker_data = None
         self.shutting_down = False
 
@@ -188,7 +194,7 @@ class Pool(Thread):
         p.stdin.flush(), p.stdin.close()
         conn = eintr_retry_call(self.listener.accept)
         w = Worker(p, conn, self.events, self.name)
-        if self.common_data != cPickle.dumps(None, -1):
+        if self.common_data != pickle_dumps(None):
             w.set_common_data(self.common_data)
         return w
 
@@ -207,7 +213,7 @@ class Pool(Thread):
         from calibre.utils.ipc.server import create_listener
         self.auth_key = os.urandom(32)
         self.address, self.listener = create_listener(self.auth_key)
-        self.worker_data = cPickle.dumps((self.address, self.auth_key), -1)
+        self.worker_data = msgpack_dumps((self.address, self.auth_key))
         if self.start_worker() is False:
             return
 
@@ -239,12 +245,12 @@ class Pool(Thread):
                 return False
             self.results.put(worker_result)
         else:
-            self.common_data = cPickle.dumps(event, -1)
+            self.common_data = pickle_dumps(event)
             if len(self.common_data) > MAX_SIZE:
                 self.cd_file = PersistentTemporaryFile('pool_common_data')
                 with self.cd_file as f:
                     f.write(self.common_data)
-                self.common_data = cPickle.dumps(File(f.name), -1)
+                self.common_data = pickle_dumps(File(f.name))
             for worker in self.available_workers:
                 try:
                     worker.set_common_data(self.common_data)
@@ -276,7 +282,7 @@ class Pool(Thread):
     def terminal_error(self):
         if self.shutting_down:
             return
-        for worker, job in self.busy_workers.iteritems():
+        for worker, job in iteritems(self.busy_workers):
             self.results.put(WorkerResult(job.id, Result(None, None, None), True, worker))
             self.tracker.task_done()
         while self.pending_jobs:
@@ -295,6 +301,7 @@ class Pool(Thread):
                     pass  # If the process has already been killed
         workers = [w.process for w in self.available_workers + list(self.busy_workers)]
         aw = list(self.available_workers)
+
         def join():
             for w in aw:
                 try:
@@ -329,12 +336,13 @@ class Pool(Thread):
             except EnvironmentError:
                 pass
 
+
 def worker_main(conn):
     from importlib import import_module
     common_data = None
     while True:
         try:
-            job = cPickle.loads(eintr_retry_call(conn.recv_bytes))
+            job = pickle_loads(eintr_retry_call(conn.recv_bytes))
         except EOFError:
             break
         except KeyboardInterrupt:
@@ -348,7 +356,9 @@ def worker_main(conn):
             break
         if not isinstance(job, Job):
             if isinstance(job, File):
-                common_data = cPickle.load(open(job.name, 'rb'))
+                with lopen(job.name, 'rb') as f:
+                    common_data = f.read()
+                common_data = pickle_loads(common_data)
             else:
                 common_data = job
             continue
@@ -368,7 +378,7 @@ def worker_main(conn):
             import traceback
             result = Result(None, as_unicode(err), traceback.format_exc())
         try:
-            eintr_retry_call(conn.send_bytes, cPickle.dumps(result, -1))
+            eintr_retry_call(conn.send_bytes, pickle_dumps(result))
         except EOFError:
             break
         except Exception:
@@ -378,15 +388,19 @@ def worker_main(conn):
             return 1
     return 0
 
+
 def run_main(func):
     from multiprocessing.connection import Client
     from contextlib import closing
-    address, key = cPickle.loads(eintr_retry_call(sys.stdin.read))
+    stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+    address, key = msgpack_loads(eintr_retry_call(stdin.read))
     with closing(Client(address, authkey=key)) as conn:
         raise SystemExit(func(conn))
 
+
 def test_write():
-    print ('Printing to stdout in worker')
+    print('Printing to stdout in worker')
+
 
 def test():
     def get_results(pool, ignore_fail=False):
@@ -394,8 +408,8 @@ def test():
         while not p.results.empty():
             r = p.results.get()
             if not ignore_fail and r.is_terminal_failure:
-                print (r.result.err)
-                print (r.result.traceback)
+                print(r.result.err)
+                print(r.result.traceback)
                 raise SystemExit(1)
             ans[r.id] = r.result
         return ans
@@ -407,7 +421,7 @@ def test():
         p(i, 'def x(i):\n return 2*i', 'x', i)
         expected_results[i] = 2 * i
     p.wait_for_tasks(30)
-    results = {k:v.value for k, v in get_results(p).iteritems()}
+    results = {k:v.value for k, v in iteritems(get_results(p))}
     if results != expected_results:
         raise SystemExit('%r != %r' % (expected_results, results))
     p.shutdown(), p.join()
@@ -421,7 +435,7 @@ def test():
         p(i, 'def x(i, common_data=None):\n return common_data + i', 'x', i)
         expected_results[i] = 7 + i
     p.wait_for_tasks(30)
-    results = {k:v.value for k, v in get_results(p).iteritems()}
+    results = {k:v.value for k, v in iteritems(get_results(p))}
     if results != expected_results:
         raise SystemExit('%r != %r' % (expected_results, results))
     p.shutdown(), p.join()
@@ -443,7 +457,7 @@ def test():
         p(i, 'def x(i):\n return 1/0', 'x', i)
     p.wait_for_tasks(30)
     c = 0
-    for r in get_results(p).itervalues():
+    for r in itervalues(get_results(p)):
         c += 1
         if not r.traceback or 'ZeroDivisionError' not in r.traceback:
             raise SystemExit('Unexpected result: %s' % r)
@@ -473,4 +487,4 @@ def test():
         p(i, 'import time;\ndef x(i):\n time.sleep(10000)', 'x', i)
     p.shutdown(), p.join()
 
-    print ('Tests all passed!')
+    print('Tests all passed!')
